@@ -47,9 +47,6 @@ TO_HEADER:
 
 
 
-/* function prototypes */
-#include "assem_conv_diff.h"
-
 /* prototypes of external functions */
 #include <math.h>
 #include "elements.h"
@@ -58,10 +55,18 @@ TO_HEADER:
 #include "mesh.h"
 #include "feins_lapack.h"    /* link against LAPACK! */
 
+/* function prototypes */
+#include "assem_conv_diff.h"
 
 /*FUNCTION*/
-int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
-		       struct mesh *m, int stab, FIDX type
+int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
+                       struct vector *rhs,  struct vector *rhsdot,
+		       double *maxPe, double *eps,
+		       struct sparse *A, struct sparse *B,
+		       double cA_K, double cA_M, 
+		       double cB_K, double cB_M, 
+		       struct mesh *m, int stab, double time,
+		       int type
 /* performs the assembly of the stiffness matrix K and the right hand
    side vector rhs which result from SUPG stabelised finite element
    discretisation of the convection-diffusion equation on the
@@ -69,25 +74,60 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 
            K x = rhs
 
+   or for time dependent problems 
+        
+           M \dot{x} + K x = rhs
+
    defines the (approximate) solution x=(u) of
 
-         -eps Laplace(u) + b * grad u = f
+     [du/dt]  -\nabla \dot eps \nabla(u) + b * grad u = f
 
-   with eps=m.para[MC2XPANUPO], b=m.function[0] and boundary conditions as
-   given in the mesh m 
+   with where eps is a matrix of diffusion coefficients,
+   b=m.function[0] and boundary conditions as given in the mesh m 
 
-   Input:  m       - the mesh
+   In addition or alternatively the matrices 
+               A= cA_K*K + cA_M*M, 
+               B= cB_K*K + cB_M*M
+   can be computed, where the coefficients c*_* are user defined. This
+   is designed in order to allow the computation of the matrices for
+   time stepping schemes, e.g. the backward Euler scheme can be
+   written as
+               (1/tau M + K) u(k+1) = b(t) + 1/tau M u(k)
+              =           A  u(k+1) = b(t) +       B u(k).            
+
+   Input:  eps     - matrix of diffusion coefficients, eps should be
+                     symmetric positive definite, eps_i,j=eps[i*dim+j]
+           m       - the mesh
 	   stab    - to indicate wether stabilization shall be used
 	             for the advection term or not, stab==0: use no
 	             stabilisation, stab==1: use SUPG stabilisation
            type    - type of mesh, ==1 means linear (P1) elements, 
-                     ==2 means quadratic (P2) elements
+                     ==2 means quadratic (P2) elements,
+           cA_K, cA_M, cB_K, cB_M
+                   - coefficients for assembling K and M to the
+                     matrices A and B, as described above. ignored if
+                     the pointer to the correspoding matrix (A or B)
+                     is ==NULL     
+	   time    - for time dependent problems this is the time at
+	             which K,M,A,B,rhs,rhsdot are to be evaluated. for
+	             stationary problems just use time=0.0
            
-   In/Out: K       - stiffness matrix as struct sparse, initialisation
+   In/Out: M       - mass matrix as struct sparse, initialisation
                      and finalisation should be done by the caling
-                     routine  
+                     routine, ignored if ==NULL
+           K       - stiffness matrix as struct sparse, initialisation
+                     and finalisation should be done by the caling
+                     routine, ignored if ==NULL
+           A       - A=cA_K*K+cA_M*M, as struct sparse, initialisation
+                     and finalisation should be done by the caling
+                     routine, ignored if ==NULL
+           B       - B=cB_K*K+cB_M*M, as struct sparse, initialisation
+                     and finalisation should be done by the caling
+                     routine, ignored if ==NULL
 	   rhs     - righthand side vector, has to be initialised by
                      the calling routine
+	   rhsdot  - time derivative of the righthand side vector, has
+	             to be initialised by the calling routine
 
    Output: maxPe   - maximum Peclet number observed in the mesh
 
@@ -95,8 +135,8 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
            FAIL    - failure, see error message, output will not be
                      valid
 */
-		){
-  FIDX el, d, i, j, k, l, r, line, col;
+		       ){
+  FIDX el, d, i, j, k, r, s;
   int  err;
 
   FIDX dim, bas_n, bas_n_2, vx_nr, eg_nr, el_nr,
@@ -104,35 +144,49 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 
   /* for FORTRAN calls: */
   char fTrans='T', fNoTrans='N';
-  double dhelp, done=1.0, dzero=0.0;
-  int fdim, fbas_n; 
+  double done=1.0, dzero=0.0;
+  int fdim, fbas_n, fbas_nc; 
 
   FIDX l_mctxVXSTRT, l_mctxEGNOD1, l_mctxELNOD1, l_mctxBDTYPE,
     l_mctxBDEDGE;
   int cubat_order;
 
-
+  double min_eps;        /* min eigenvalue of matrix eps */
 
   struct int_data iform; /* integration formula 2d   */
-  double *x_k;            /* the integration point */
   double *bk;            /* velocity vector at the integration point */
-  double fx_k;           /* rhs function f(.) at the integration point */
+  double fxc;            /* rhs function f(.) at the integration point */
+  double *dotfxc;        /* time derivative of rhs function f(.) at
+			    the integration point */
+  double dotfxc_mem;     /* memory in case dotfxc is used */
   double *elK;           /* element stiffness matrix */
+  double *elM;           /* element mass matrix */
+  double *elAB;          /* temporary storage for cA_M*elM and the
+			    other c?_?*el? */
   double *Jac, *Jacinv;  /* Jacobian of the element mapping and its
 			    inverse */
   double *Jinvgrad;      /* inverse Jacobian times gradphi */
+  double *Hessglob;      /* real world Hessian */
   double detJac;         /* determinant of the Jacobian */
 
-  double *phi, *gradphi;
-  double eps, weight;
+  FIDX   m_type;
+  double *phi, *gradphi, *hessphi;
+  double weight;
 
   double *xc;            /* the center point of the element */
   double *bc;            /* velocity at the element center */
   double h, abs_bc, heg, h_max, hbc, hbcmax, pe, beta;
                          /* values used to determine the mesh peclet
 			    number and the stabilization term */ 
-  double *phic, *gphic, *hphic;
-                         /* the basis function values at this point */
+  int    bas_nc;         /* number of linear basis functions (used in xc)*/
+  double *phic, *gphic, *hphic, *gradc_glob; 
+                         /* the basis function for linear elements
+                            values at this point */ 
+
+  int computeMatrices;
+
+  computeMatrices= ( ((A!=NULL) || (B!=NULL)) || ((M!=NULL) || (K!=NULL)) );
+    
 
   /****************   init ******************************************/
   dim   = (*m).dim;
@@ -146,7 +200,7 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
   fu_w  = (*m).fu_w;
   fc_w  = (*m).fc_w;
 
-  eps= (*m).para[MC2XPANUPO];
+
 
   /* get integration formula */
   /* required order: 
@@ -189,19 +243,19 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
       return FAIL;
     }
 
-  err=cubature_bases( dim, cubat_order, tria, 1, &type, &iform); 
+  m_type=(FIDX) type;
+  err=cubature_bases( dim, cubat_order, tria, 1, &m_type, &iform); 
   FUNCTION_FAILURE_HANDLE( err, cubature_bases, assem_conv_diff_tx);
 
   /* make phi and gradphi better accessible */
   phi    = (iform.bases[0]->phi);
   gradphi= (iform.bases[0]->gradphi);
+  hessphi= (iform.bases[0]->hessphi);
   bas_n  = (iform.bases[0]->num_basis);
 
   fbas_n = (int) bas_n;
   fdim   = (int) dim;
 
-
-
   if (dim > 2)
     {
       /* cry */
@@ -210,30 +264,70 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
       return FAIL;
     }
 
-  if ( (*K).row_nr!=vx_nr )
+  if ( ((K!=NULL)&&((*K).row_nr!=vx_nr))
+       ||((M!=NULL)&&((*M).row_nr!=vx_nr))
+       ||((A!=NULL)&&((*A).row_nr!=vx_nr)) 
+       ||((B!=NULL)&&((*B).row_nr!=vx_nr))
+       )
     {
       /* cry */
       fprintf(stderr,
-	      "assem_conv_diff_tx: size of K wrong???\n");
+	      "assem_conv_diff_tx: size of K,M,A,B wrong???\n");
       return FAIL; 
     }
     
+  
+  /* check that diffusion matrix eps is symmetric positive definite */
+  /* first symmetry */
+  for (i=0; i<dim; i++)
+    for (j=i+1; j<dim; j++)
+      {
+	if (eps[i*dim+j] != eps[j*dim+i])
+	  {
+	    fprintf(stderr,
+		    "assem_conv_diff_tx: eps not symmetric???\n");
+	    return FAIL; 
+	  }
+      }
+  {
+    /* check that diffusion matrix eps is positive definite */
+    char fJOBZ='N';
+    char fUPLO='U';
+    int lwork=(dim+2)*dim, fdim=dim;
+    double *work, *evs;
+    int info;
+    
+    TRY_MALLOC( work, lwork, double, assem_conv_diff_tx);
+    TRY_MALLOC( evs, dim, double, assem_conv_diff_tx);
+    dsyev_( &fJOBZ, &fUPLO, &fdim, eps, &fdim, 
+	    evs, work, &lwork, &info );
+    if (info==0)
+      { 
+	/* printf("evs= %e %e \n", evs[0], evs[1]); */
+	min_eps=evs[0];
+      }
+    else
+      {
+	fprintf(stderr,
+		"assem_conv_diff_tx: error determining eigenvalues of eps\n");
+	return FAIL; 
+      }
+    free(evs);
+    free(work);
+
+  }
+  if (min_eps<=0)
+    {
+      fprintf(stderr,
+	      "assem_conv_diff_tx: eps min eigenvalue <=0\n");
+      return FAIL; 
+    }
+
 
   bas_n_2=bas_n*bas_n;    /* bas_n^2 */
 
-  /* allocate memory for local data */
-  TRY_MALLOC( x_k, dim, double, assem_conv_diff_tx);
-  TRY_MALLOC( bk, dim, double, assem_conv_diff_tx);
+
   TRY_MALLOC( xc, dim, double, assem_conv_diff_tx);
-  TRY_MALLOC( bc, dim, double, assem_conv_diff_tx);
-  TRY_MALLOC( elK, bas_n_2, double, assem_conv_diff_tx);
-
-  TRY_MALLOC( Jac, dim*dim, double, assem_conv_diff_tx);
-  TRY_MALLOC( Jacinv, dim*dim, double, assem_conv_diff_tx);
-
-  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_conv_diff_tx);
-
-
   /* the center of gravity of the master element is
 		     (1/dim,...,1/dim) */
   xc[0]=1.0/( (double)dim);  
@@ -242,19 +336,76 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
       xc[i]=xc[0];
     }
   /* evaluate the basis functions for this point */
-  err=eval_basis( dim, tria, type, 1, xc, &i, &phic, &gphic, &hphic);
+  err=eval_basis( dim, tria, 1, 1, xc, &i, &phic, &gphic, &hphic);
   FUNCTION_FAILURE_HANDLE( err, eval_basis, assem_conv_diff_tx);
   /* free unneeded info */
   free(hphic);
+  bas_nc=3;
+
+
+
+  /* allocate memory for local data */
+  TRY_MALLOC( bk, dim, double, assem_conv_diff_tx);
+  TRY_MALLOC( bc, dim, double, assem_conv_diff_tx);
+  if (computeMatrices)
+    {
+      TRY_MALLOC( elK, bas_n_2, double, assem_conv_diff_tx);
+      TRY_MALLOC( elM, bas_n_2, double, assem_conv_diff_tx);
+      TRY_MALLOC( elAB, bas_n_2, double, assem_conv_diff_tx);
+      TRY_MALLOC( Hessglob, bas_n*dim*dim, double, assem_conv_diff_tx);
+    }
+  else
+    { 
+      elK     =NULL;
+      elM     =NULL;
+      elAB    =NULL;
+      Hessglob=NULL;
+    }
+
+  TRY_MALLOC( Jac, dim*dim, double, assem_conv_diff_tx);
+  TRY_MALLOC( Jacinv, dim*dim, double, assem_conv_diff_tx);
+
+  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_conv_diff_tx);
+
+  TRY_MALLOC( gradc_glob, dim*bas_nc, double, assem_conv_diff_tx);
+
+  /* clear K,M,A,B */
+  if (K!=NULL)
+    {
+      sparse_empty(K);
+    }
+  if (M!=NULL)
+    {
+      sparse_empty(M);
+    }
+  if (A!=NULL)
+    {
+      sparse_empty(A);
+    }
+  if (B!=NULL)
+    {
+      sparse_empty(B);
+    }
 
-  /* clear K */
-  sparse_empty(K);
   /* clear rhs */
   for (i=0; i<vx_nr; i++)
     {
       rhs->V[i]=0.0;
     }
 
+  /* clear rhsdot */
+  if (rhsdot!=NULL)
+    {
+      for (i=0; i<vx_nr; i++)
+	{
+	  rhsdot->V[i]=0.0;
+	}
+      dotfxc=&dotfxc_mem;
+    }
+  else
+    {
+      dotfxc=NULL;
+    }
   *maxPe=0.0;
 
   /* loop over all elements */
@@ -266,13 +417,13 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
       for (j=0; j<dim; j++)
 	{
 	  xc[j]=0.0;
-	  for (i=0; i<bas_n; i++)
+	  for (i=0; i<bas_nc; i++)
 	    xc[j]+=phic[i]
 	      *(*m).vertex[(*m).elem[el*el_w+l_mctxELNOD1+i]*vx_w
 			   +l_mctxVXSTRT+j];
 	}
       /* velocity always given by b=m.function[0] */
-      err=mesh_func_eval( m, 0, xc, dim, bc);
+      err=mesh_func_eval( m, 0, xc, time, dim, bc, NULL, NULL);
       FUNCTION_FAILURE_HANDLE( err, mesh_func_eval, assem_conv_diff_tx);
       
       /* determine norm of the velocity vector */
@@ -281,41 +432,68 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 	abs_bc+=bc[j]*bc[j];
       abs_bc=sqrt(abs_bc);
 
-      /* now get the length of the edge which coincides best with the
-	 direction of bc */
-      hbcmax=-1.0;
-      h_max=-1.0;
-      for (i=0; i<bas_n; i++)
-	for (j=i+1; j<bas_n; j++)
-	  {
-#warning "assem_conv_diff: use better approximation of cell size for stabilization!"
-	    FIDX nodei, nodej;
-	    nodei=(*m).elem[el*el_w+l_mctxELNOD1+i];
-	    nodej=(*m).elem[el*el_w+l_mctxELNOD1+j];
-	    hbc=0.0;
-	    for(d=0; d<dim; d++)
-	      hbc+=bc[d]*( (*m).vertex[nodei*vx_w+l_mctxVXSTRT+d]
-			   -(*m).vertex[nodej*vx_w+l_mctxVXSTRT+d] );
-		
-	    heg=0.0; /* h_edge */
-	    for(d=0; d<dim; d++)
-	      heg+=( (*m).vertex[nodei*vx_w+l_mctxVXSTRT+d]
-		     -(*m).vertex[nodej*vx_w+l_mctxVXSTRT+d] )
-		*( (*m).vertex[nodei*vx_w+l_mctxVXSTRT+d]
-		   -(*m).vertex[nodej*vx_w+l_mctxVXSTRT+d] );
-	    heg=sqrt(heg);
-		
-	    if (heg>h_max) h_max=heg; /* max(h_edge) */
+      /* get the length of the element in stream-wise direction */
+      /*   %% diameter of element in direction btil (Petr Knobloch)
+	   %hmax2=0;
+	   %for j=1:3
+	   %  hmax2=hmax2+abs(b'*gradglob(:,j));
+	   %end
+	   %h=2*norm(b)/hmax2;
+      */
+
+      /* first compute the gradient of the linear ansatz functions */
+      /* compute the Jacobian for this element */
+      /* Jac=0 */
+      for (i=0; i<dim*dim; i++)
+	Jac[i]=0.0;
+	  
+      /* Jac = sum_{i=nodes} vertex(i)*gradphi_i^T */
+      for (i=0;i<bas_nc; i++)
+	{
+	  for (j=0; j<dim; j++)
+	    {
+	      for (r=0; r<dim; r++)
+		{
+		  Jac[j*dim+r]+= 
+		    (*m).vertex[(*m).elem[el*el_w+l_mctxELNOD1+i]*vx_w
+				+l_mctxVXSTRT+j]
+		    * gphic[0*bas_nc*dim +i*dim +r];
+		}
+	    }
+	}
+
+      /* get detJac */
+      detJac=Jac[0]*Jac[3]-Jac[2]*Jac[1];
+	  
+      /* get Jacinv (here direct) */
+      Jacinv[0]=1.0/detJac*Jac[3];
+      Jacinv[1]=-1.0/detJac*Jac[1];
+      Jacinv[2]=-1.0/detJac*Jac[2];
+      Jacinv[3]=1.0/detJac*Jac[0];
+
+
+      /* gradc_glob= Jacinv * gradphi[k,:,:] 
+	 (=real world gradient T1)
+      */
+      fbas_nc=bas_nc;
+      dgemm_( &fNoTrans, &fNoTrans, &fdim, &fbas_nc, &fdim,
+	      &done, Jacinv, &fdim, &(gphic[0*bas_nc*dim]), &fdim,
+	      &dzero, gradc_glob, &fdim );
+
+      h_max=0.0;
+      for (i=0; i<bas_nc; i++)
+	{
+	  hbcmax=0.0;
+	  for(d=0; d<dim; d++)
+	    {
+	      hbcmax+=bc[d]*gradc_glob[i*dim+d];	  
+	    }
+	  h_max+=fabs(hbcmax);
+	}
+      h=2*abs_bc/h_max;
 
-	    hbc=fabs(hbc);
-	    if (hbc>hbcmax)         /* max(h_edge) in direction of b */
-	      {
-		hbcmax=hbc;
-		h=heg;
-	      }
-	  }
       /* this elements Peclet number Pe */
-      pe=0.5*h*abs_bc/eps;
+      pe=0.5*h*abs_bc/min_eps;
       /* update the maximum Pe */
       if (*maxPe<pe) *maxPe=pe;
       /* stabilization parameter beta */
@@ -323,7 +501,20 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 	{
 #ifdef RAMAGE_NOT_TEZDUYAR
 	  /* Ramage stab parameter */
-	  beta=(0.5*h-eps)/abs_bc;
+	  //beta=(0.5*h-min_eps)/abs_bc;
+	  /* John/Novo */
+	  //beta=sqrt(1.0/cB_M)*h/(4*abs_bc);
+
+	  beta=0.0;
+	  /* in ROS3P with adaptive timestepping, the John/Novo
+	     parameter choice appears  to generate more oszillations
+	     than beta=0, and also restricts the timesteps further
+
+	     Ramage appears to be neither advantage nor disadvantage
+	     compared to beta=0, because sufficient diffusion is
+	     present so stabilization switches of
+	  */
+	  //printf("beta=%e\n",beta);
 #else
 	  /* Tezduyar stab parameter */
 	  beta=(0.5*h/abs_bc)*fmin(1.0, 0.33333333*pe);
@@ -336,10 +527,14 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
       if ((beta<0.0)||(stab!=1)) beta=0.0;
 
 
-      /* set elK to zero */
-      for (i=0; i<bas_n_2; i++) 
+      if (computeMatrices)
 	{
-	  elK[i]=0.0;
+	  /* set elK and elM to zero */
+	  for (i=0; i<bas_n_2; i++) 
+	    {
+	      elK[i]=0.0;
+	      elM[i]=0.0;
+	    }
 	}
 
       /* loop over all integration points */
@@ -374,6 +569,8 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 	  Jacinv[2]=-1.0/detJac*Jac[2];
 	  Jacinv[3]=1.0/detJac*Jac[0];
 
+	  /* the real wolrd weight of this integration point */
+	  weight=fabs(detJac) * iform.weights[k];
 
 	  /* Jinvgrad= Jacinv * gradphi[k,:,:] 
 	     (=real world gradient T1)
@@ -382,16 +579,50 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 		  &done, Jacinv, &fdim, &(gradphi[k*bas_n*dim]), &fdim,
 		  &dzero, Jinvgrad, &fdim );
 
-
-	  /* the real wolrd weight of this integration point */
-	  weight=fabs(detJac) * iform.weights[k];
-
-	  dhelp=weight*eps;
-	  /* elK += |detJac|*weigth[k] * eps* Jinvgrad^T*Jinvgrad */
-	  dgemm_( &fTrans, &fNoTrans, &fbas_n, &fbas_n, &fdim,
-		  &dhelp, Jinvgrad, &fdim, Jinvgrad, &fdim,
-		  &done, elK, &fbas_n );
-
+	  if (computeMatrices)
+	    {
+	      /* real world Hessian (neglecting second derivatives of the
+		 element mapping) */
+	      for (i=0;i<bas_n; i++)
+		{
+		  int kk,mm;
+		  
+		  for (r=0; r<dim; r++) 
+		    for (s=0; s<dim; s++)
+		      {
+			Hessglob[i*dim*dim+r*dim+s]=0.0;
+		      }
+		  for (s=0; s<dim; s++)
+		    for (kk=0; kk<dim; kk++) 
+		      {
+			double y_ks=0.0;
+			for (mm=0; mm<dim; mm++) 
+			  {
+			    y_ks+=(hessphi[k*bas_n*dim*dim+i*dim*dim
+					   + kk*dim + mm]
+				   * Jacinv[mm*dim+s]);
+			  }
+			
+			for (r=0; r<dim; r++) 
+			  {
+			    Hessglob[i*dim*dim+r*dim+s]+=
+			      Jacinv[kk*dim+r]*y_ks;
+			  }
+		      }
+		}
+	      /* elK += |detJac|*weigth[k] * Jinvgrad^T*eps*Jinvgrad */
+	      for (i=0; i<bas_n; i++) /* row */
+		for (j=0; j<bas_n; j++) /* col */
+		  for (r=0; r<dim; r++) /* index for Jinvgrad^T*y */
+		    {
+		      double y_r=0.0;
+		      for (s=0; s<dim; s++) /* index for y=(eps*Jinvgrad) */
+			{
+			  y_r += eps[r*dim+s] * Jinvgrad[j*dim+s];
+			}
+		      elK[i*bas_n+j] += Jinvgrad[i*dim+r]*y_r * weight;
+		    }
+	    }
 
 
 	  /* define bk[], velocity vector of convection term */
@@ -404,15 +635,16 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 			       +l_mctxVXSTRT+j];
 	    }
 	  /* velocity always given by b=m.function[0] */
-	  err=mesh_func_eval( m, 0, xc, dim, bk);
+	  err=mesh_func_eval( m, 0, xc, time, dim, bk, NULL, NULL);
 	  FUNCTION_FAILURE_HANDLE( err, mesh_func_eval, assem_conv_diff_tx);
 
 
-	  /* evaluate rhs function f(x_k) */
+	  /* evaluate rhs function f(xc) (and if required its time derivative) */
 	  err=mesh_func_eval ( m, (*m).face[el*fc_w+MCT1FCRHSF],
-			       x_k, 1, &fx_k);
+			       xc, time, 1, &fxc, NULL, dotfxc);
 	  FUNCTION_FAILURE_HANDLE(err,mesh_func_eval, assem_conv_diff_tx);
 
+
 	  /* elK(i,j) += int(element)(
 	            sum_d ( bk[d]* (d phi_j/d x_d)) 
 	            * (phi_i + beta* sum_d (bk[d] *d phi_i/d x_d)))
@@ -431,18 +663,46 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
 	      for (j=0; j<bas_n; j++)
 		{
 		  double b_gradphi_j=0.0;
+		  double eps_laplace_gradphi_j=0.0;
+
 		  for (d=0; d<dim; d++)
 		    {
 		      b_gradphi_j+=bk[d]*Jinvgrad[j*dim+d];
 		    }
-		  elK[j*bas_n+i] +=
-		    b_gradphi_j *( phi[k*bas_n+i] + beta*b_gradphi_i)
-		    * weight ; /* */
+
+
+		  if (computeMatrices)
+		    {
+		      for (r=0; r<dim; r++) 
+			for (s=0; s<dim; s++)
+			  {
+			    eps_laplace_gradphi_j+=
+			      eps[r*dim+s]*Hessglob[j*dim*dim+r*dim+s];
+			  }
+		      
+		      elK[j*bas_n+i] +=
+			( (b_gradphi_j * phi[k*bas_n+i])
+			  +(-eps_laplace_gradphi_j+b_gradphi_j)* beta*b_gradphi_i)
+			* weight ; /* */
+		      
+		      elM[j*bas_n+i] +=
+			( phi[k*bas_n+j]
+			  *(phi[k*bas_n+i])+ beta*b_gradphi_i)
+			* weight ; /* */
+		    }
 		}
 	      /* compute rhs */
 	      (*rhs).V[(*m).elem[el*el_w+i]] +=
-		fx_k *( phi[k*bas_n+i] + beta*b_gradphi_i)
+		fxc *( phi[k*bas_n+i] + beta*b_gradphi_i)
 		* weight ;
+
+	      /* compute rhsdot */
+	      if (rhsdot!=NULL)
+		{
+		  (*rhsdot).V[(*m).elem[el*el_w+i]] +=
+		    (*dotfxc) *( phi[k*bas_n+i] + beta*b_gradphi_i)
+		    * weight ;
+		}
 	    }
 	} /* end loop intergration points */
       
@@ -450,27 +710,99 @@ int assem_conv_diff_tx(struct sparse *K, struct vector *rhs, double *maxPe,
        * elK is ready, now assemble K                                *
        ***************************************************************/
 
-      err=sparse_add_local( K, NoTrans,
-			   bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
-			   bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
-			   elK, bas_n );
-      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+      if (K!=NULL)
+	{
+	  err=sparse_add_local( K, NoTrans,
+				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				elK, bas_n );
+	  FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	}
+
+      if (M!=NULL)
+	{
+	  err=sparse_add_local( M, NoTrans,
+				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				elM, bas_n );
+	  FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	}
+
+      if (A!=NULL)
+	{
+	  if (cA_M!=0.0)
+	    {
+	      for (i=0; i<bas_n_2; i++)
+		{
+		  elAB[i] = cA_M*elM[i];
+		}
+	      err=sparse_add_local( A, NoTrans,
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				    elAB, bas_n );
+	      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	    }
+	  if (cA_K!=0.0)
+	    {
+	      for (i=0; i<bas_n_2; i++)
+		{
+		  elAB[i] = cA_K*elK[i];
+		}
+	      err=sparse_add_local( A, NoTrans,
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				    elAB, bas_n );
+	      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	    }
+	}
+      if (B!=NULL)
+	{
+	  if (cB_M!=0.0)
+	    {
+	      for (i=0; i<bas_n_2; i++)
+		{
+		  elAB[i] = cB_M*elM[i];
+		}
+	      err=sparse_add_local( B, NoTrans,
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				    elAB, bas_n );
+	      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	    }
+	  if (cB_K!=0.0)
+	    {
+	      for (i=0; i<bas_n_2; i++)
+		{
+		  elAB[i] = cB_K*elK[i];
+		}
+	      err=sparse_add_local( B, NoTrans,
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				    bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				    elAB, bas_n );
+	      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	    }
+	}
 
     } /* end loop elements */
 
-  /* printf("lvl=%d,  maxPe=%8.1e\n", (int) lvl, *maxPe); /* */
+  /* printf("lvl=%"dFIDX",  maxPe=%8.1e\n",  lvl, *maxPe); /* */
 
   /* free local data */
-  free(phic);
-  free(gphic);
+  free(gradc_glob);
+  free(Hessglob);
   free(Jinvgrad);
   free(Jacinv);
   free(Jac);
   free(elK);
+  free(elM);
+  free(elAB);
   free(bc);
   free(xc);
   free(bk);
-  free(x_k);
+
+  free(phic);
+  free(gphic);
+
   free_intdata (&iform);
 
 
