@@ -47,16 +47,20 @@ TO_HEADER:
 
 
 
-/* function prototypes */
-#include "assembly.h"
 
 /* prototypes of external functions */
+#define _USE_MATH_DEFINES 
 #include <math.h>
 #include "cubature.h"
+#include "elements.h"
 #include "sparse.h"
 #include "mesh.h"
+#include "gen_aux.h"
 #include "feins_lapack.h"    /* link against LAPACK! */
 
+/* function prototypes */
+#include "assembly.h"
+
 /*FUNCTION*/
 int assem_poison_t1(struct sparse *K, struct vector *rhs,
 		    struct vector *u0, struct projector1 *P, 
@@ -224,7 +228,7 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
              	       = sum weight_k*|detJac| * f(x_k) * phi(x_k) */
 	      /* get f(x_k) */
 	      err=mesh_func_eval ( m, (*m).face[el*fc_w+MCT1FCRHSF],
-				   x_k, 1, &fx_k);
+				   x_k, 0.0, 1, &fx_k, NULL, NULL);
 	      FUNCTION_FAILURE_HANDLE(err,mesh_func_eval,
 				      assem_poison_t1);
 	      for (i=0; i<bas_n; i++)
@@ -247,7 +251,9 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
     }
   /* end loop over all elements */
 
-#warning poison_T1: Neumann BC not accounted for in rhs yet!
+#ifdef FEINS_have_warning 
+#warning "poison_T1: Neumann BC not accounted for in rhs yet!"
+#endif
   /* first Neumann BC, otherwise they would collide with Dirichlet BC
   */
 
@@ -269,10 +275,10 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
 
 	  err1=mesh_func_eval(m, fu,
 			      &(*m).vertex[node1*vx_w+MCT1VXSTRT],
-			      1, &(*u0).V[node1]);
+			      0.0, 1, &(*u0).V[node1], NULL, NULL);
 	  err2=mesh_func_eval(m, fu,
 			      &(*m).vertex[node2*vx_w+MCT1VXSTRT],
-			      1, &(*u0).V[node2]);
+			      0.0, 1, &(*u0).V[node2], NULL, NULL);
 	  if ((err1!=SUCCESS)||(err2)!=SUCCESS)
 	    {
 	      fprintf(stderr,
@@ -492,7 +498,7 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
              	       = sum weight_k*|detJac| * f(x_k) * phi(x_k) */
 	      /* get f(x_k) */
 	      err=mesh_func_eval ( m, (*m).face[el*fc_w+MCT2FCRHSF],
-				   x_k, 1, &fx_k);
+				   x_k, 0.0, 1, &fx_k, NULL, NULL);
 	      FUNCTION_FAILURE_HANDLE(err,mesh_func_eval,
 				      assem_poison_t2);
 	      for (i=0; i<bas_n; i++)
@@ -516,8 +522,9 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
 		       
     }
   /* end loop over all elements */
-
-#warning poison_T2: Neumann BC not accounted for in rhs yet!
+#ifdef FEINS_have_warning 
+#warning "poison_T2: Neumann BC not accounted for in rhs yet!"
+#endif
   /* first Neumann BC, otherwise they would collide with Dirichlet BC
   */
 
@@ -540,13 +547,13 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
 
 	  err1=mesh_func_eval(m, fu,
 			      &(*m).vertex[node1*vx_w+MCT2VXSTRT],
-			      1, &(*u0).V[node1]);
+			      0.0, 1, &(*u0).V[node1], NULL, NULL);
 	  err2=mesh_func_eval(m, fu,
 			      &(*m).vertex[node2*vx_w+MCT2VXSTRT],
-			      1, &(*u0).V[node2]);
+			      0.0, 1, &(*u0).V[node2], NULL, NULL);
 	  errm=mesh_func_eval(m, fu,
 			      &(*m).vertex[nodem*vx_w+MCT2VXSTRT],
-			      1, &(*u0).V[nodem]);
+			      0.0, 1, &(*u0).V[nodem], NULL, NULL);
 	  if (((err1!=SUCCESS)||(err2)!=SUCCESS)||((errm)!=SUCCESS))
 	    {
 	      fprintf(stderr,
@@ -682,6 +689,18 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
   /* loop over all elements */
   for (el=0; el<(*m).el_nr; el++)
     {
+#ifdef DEBUGFEINS
+      for (i=0;i<bas_n; i++)
+	{
+	  if ((*m).elem[el*el_w+MCT2ELNOD1+i]>=vx_nr1)
+	    {
+	      fprintf(stderr,"assem_poison_t1_t2: error, element contains\n"
+		      "vertex %d > vx_nr1=%d\n",
+		      (int) (*m).elem[el*el_w+MCT2ELNOD1+i], (int) vx_nr1);
+	      return FAIL;
+	    }
+	}
+#endif
       /* set elK to zero */
       for (i=0; i<bas_n*bas_n; i++) 
 	{
@@ -796,12 +815,15 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
 
 
 /*FUNCTION*/
-int assem_mass_t1(struct sparse *M,  
-		  struct mesh *m
+int assem_mass_tx(struct sparse *M,  
+		  struct mesh *m,
+		  FIDX type
 /* performs the assembly of the mass matrix M for finite element
    discretisations 
 
    Input:  m       - the mesh
+           type    - type of mesh, ==1 means linear (P1) elements, 
+                     ==2 means quadratic (P2) elements,
 
    Output: M       - mass matrix
 
@@ -815,10 +837,12 @@ int assem_mass_t1(struct sparse *M,
 
   FIDX dim=2, bas_n, vx_nr, el_w, vx_w;
   FIDX subtypes[1];
+  FIDX l_mctxVXSTRT, l_mctxELNOD1;
+  int cubat_order;
+
 
   /* for FORTRAN calls: */
-  char fTrans='T', fNoTrans='N';
-  double dhelp, done=1.0, dzero=0.0;
+  double dhelp;
   int fdim, fbas_n;
 
   struct int_data iform; /* integration formula    */
@@ -830,16 +854,26 @@ int assem_mass_t1(struct sparse *M,
 
   /* init */
   /* get integration formula */
-  subtypes[0]=1;
-
-  err=cubature_bases( dim, 2, tria, 1, subtypes, &iform); 
-
-  if (err!=SUCCESS)
+  switch(type)
     {
-      fprintf( stderr,
-	       "cubature_bases returned error in assem_mass_t1!\n");
+    case 1:
+      l_mctxVXSTRT = MCT1VXSTRT;
+      l_mctxELNOD1 = MCT1ELNOD1;
+      cubat_order=4;
+      break;
+    case 2:
+      l_mctxVXSTRT = MCT2VXSTRT;
+      l_mctxELNOD1 = MCT2ELNOD1;
+      cubat_order=7;
+      break;
+    default:
+      fprintf(stderr,"assem_mass_tx: unknown type=%d\n", (int) type);
       return FAIL;
     }
+  subtypes[0]=type;
+
+  err=cubature_bases( dim, cubat_order, tria, 1, subtypes, &iform); 
+  FUNCTION_FAILURE_HANDLE( err, cubature_bases, assem_mass_tx);
 
   /* make phi and gradphi better accessible */
   phi   = (iform.bases[0]->phi);
@@ -854,10 +888,10 @@ int assem_mass_t1(struct sparse *M,
   el_w  = (*m).el_w;
 
   /* allocate memory for elM */
-  TRY_MALLOC( elM, bas_n*bas_n, double, assem_mass_t1);
+  TRY_MALLOC( elM, bas_n*bas_n, double, assem_mass_tx);
   /* allocate memory for Jac, Jacinf */
-  TRY_MALLOC( Jac, dim*dim, double, assem_mass_t1);
-  TRY_MALLOC( Jacinv, dim*dim, double, assem_mass_t1);
+  TRY_MALLOC( Jac, dim*dim, double, assem_mass_tx);
+  TRY_MALLOC( Jacinv, dim*dim, double, assem_mass_tx);
 
   /* clear M */
   sparse_empty(M);
@@ -887,8 +921,8 @@ int assem_mass_t1(struct sparse *M,
 		  for (r=0; r<dim; r++)
 		    {
 		      Jac[j*dim+r]+= 
-			(*m).vertex[(*m).elem[el*el_w+MCT1ELNOD1+i]*vx_w
-				    +MCT1VXSTRT+j]
+			(*m).vertex[(*m).elem[el*el_w+l_mctxELNOD1+i]*vx_w
+				    +l_mctxVXSTRT+j]
 			* gradp[k*bas_n*dim +i*dim +r];
 		    }
 		}
@@ -921,10 +955,10 @@ int assem_mass_t1(struct sparse *M,
       
       /* elM is ready, add to M */
       err=sparse_add_local(M, NoTrans,
-			   bas_n, &((*m).elem[el*el_w+MCT1ELNOD1]),
-			   bas_n, &((*m).elem[el*el_w+MCT1ELNOD1]),
+			   bas_n, &((*m).elem[el*el_w+l_mctxELNOD1]),
+			   bas_n, &((*m).elem[el*el_w+l_mctxELNOD1]),
 			   elM, bas_n );
-      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_mass_t1);
+      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_mass_tx);
 	       
     }
   /* end loop over all elements */
@@ -933,7 +967,7 @@ int assem_mass_t1(struct sparse *M,
     {
       err = sparse_convert_compressed_row( M );
       FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
-			       assem_mass_t1 );
+			       assem_mass_tx );
     }
 
   /* free local data */
@@ -1437,7 +1471,7 @@ int assem_lame_tx(struct sparse *K, struct projector1 *P, struct mesh *msh, stru
 	        /* NODE 1 Fct-eval and copying to nodem.V */
 	        err=mesh_func_eval(&(*msh), fu,
 			           &(*msh).vertex[node1*(*msh).vx_w+MCT2VXSTRT],
-			           dim, feval_result);
+			           0.0, dim, feval_result, NULL, NULL);
 
 	        if ((err)!=SUCCESS)
 	          {
@@ -1454,7 +1488,7 @@ int assem_lame_tx(struct sparse *K, struct projector1 *P, struct mesh *msh, stru
 	        /* NODE 2 Fct-eval and copying to nodem.V */
 	        err=mesh_func_eval(&(*msh), fu,
 		  	           &(*msh).vertex[node2*(*msh).vx_w+MCT2VXSTRT],
-			           dim, feval_result);
+			           0.0, dim, feval_result, NULL, NULL);
 
 	        if ((err)!=SUCCESS)
 	          {
@@ -1472,7 +1506,7 @@ int assem_lame_tx(struct sparse *K, struct projector1 *P, struct mesh *msh, stru
 	        /* NODE 3 Fct-eval and copying to nodem.V */
                 err=mesh_func_eval(&(*msh), fu, 
 			           &(*msh).vertex[node3*(*msh).vx_w+MCT2VXSTRT],
-			           dim, feval_result);
+			           0.0, dim, feval_result, NULL, NULL);
 
 	        if ((err)!=SUCCESS)
 	          {
@@ -1588,13 +1622,6 @@ int assem_lame_tx_tx(struct sparse *K, struct projector1 *P,
     vx_nr, el_w, vx_w, eg_w, fc_w, bd_w, fu_w;
   FIDX subtypes[1];
 
-  if ((*msh).dim!=2)
-    {
-      fprintf( stderr,
-	       "assem_lame_tx is only implemented for 2D meshes!\n");
-      return FAIL;
-    }			    
-			    
   /* for FORTRAN calls: */
   char   fNoTrans='N';
   double dhelp, done=1.0, dzero=0.0;
@@ -1612,6 +1639,14 @@ int assem_lame_tx_tx(struct sparse *K, struct projector1 *P,
 
   FIDX l_mctxELNOD1, l_mctxVXSTRT, l_mctxBDTYPE, l_mctxBDEDGE, l_mctxEGNOD1;
 
+  if ((*msh).dim!=2)
+    {
+      fprintf( stderr,
+	       "assem_lame_tx is only implemented for 2D meshes!\n");
+      return FAIL;
+    }			    
+			    
+
   switch (type)
     {
     case 1:
@@ -1638,12 +1673,7 @@ int assem_lame_tx_tx(struct sparse *K, struct projector1 *P,
   subtypes[0]=type;
 
   err=cubature_bases( dim, 4, tria, 1, subtypes, &iform); 
-  if (err!=SUCCESS)
-    {
-      fprintf( stderr,
-	       "cubature_bases returned error in assem_lame_tx!\n");
-      return FAIL;
-    }
+  FUNCTION_FAILURE_HANDLE( err, cubature_bases, assem_lame_tx);
 
   /* make phi and gradphi better accessible */
   phi   = (iform.bases[0]->phi);
@@ -1879,7 +1909,7 @@ int assem_lame_tx_tx(struct sparse *K, struct projector1 *P,
 	        /* NODE 1 Fct-eval and copying to nodem.V */
 	        err=mesh_func_eval(&(*msh), fu,
 			           &(*msh).vertex[node1*(*msh).vx_w+MCT2VXSTRT],
-			           dim, feval_result);
+			           0.0, dim, feval_result, NULL, NULL);
 	        if ((err)!=SUCCESS)
 	          {
 	            fprintf(stderr,
@@ -1896,7 +1926,7 @@ int assem_lame_tx_tx(struct sparse *K, struct projector1 *P,
 	        /* NODE 2 Fct-eval and copying to nodem.V */
 	        err=mesh_func_eval(&(*msh), fu,
 		  	           &(*msh).vertex[node2*(*msh).vx_w+MCT2VXSTRT],
-			           dim, feval_result);
+			           0.0, dim, feval_result, NULL, NULL);
 	        if ((err)!=SUCCESS)
 	          {
 	            fprintf(stderr,
@@ -1914,7 +1944,7 @@ int assem_lame_tx_tx(struct sparse *K, struct projector1 *P,
 	            /* NODE 3 Fct-eval and copying to nodem.V */
                     err=mesh_func_eval(&(*msh), fu, 
 			           &(*msh).vertex[node3*(*msh).vx_w+MCT2VXSTRT],
-			           dim, feval_result);
+				       0.0, dim, feval_result, NULL, NULL);
 
 	            if ((err)!=SUCCESS)
 	              {
@@ -2065,7 +2095,7 @@ int lame_neum_bc_t2( struct mesh *m, struct vector *rhs
 		    }
 		}
 
-	      err=mesh_func_eval(m, fu, x, 2, g );
+	      err=mesh_func_eval(m, fu, x, 0.0, 2, g, NULL, NULL );
 	      if (err!=SUCCESS)
 		{
 		  fprintf(stderr, "lame_neum_bc_t2: "
@@ -2155,9 +2185,8 @@ int assem_Dirichlet_BC_tx(struct projector1 *P, struct mesh *msh,
 */
 		    ){
   FIDX bd,i,j,r,s;
-  int err;
 
-  FIDX dim=2, vx_nr, vx_w, eg_w, bd_w, fu_w;
+  FIDX vx_nr, vx_w, eg_w, bd_w, fu_w;
 
 
   FIDX l_mctxVXSTRT, l_mctxBDTYPE, l_mctxBDEDGE, l_mctxBDFNCT, l_mctxEGNOD1;
@@ -2224,7 +2253,7 @@ int assem_Dirichlet_BC_tx(struct projector1 *P, struct mesh *msh,
 	  /* NODE 1 Fct-eval and copying to nodem.V */
 	  err=mesh_func_eval(&(*msh), fu,
 			     &(*msh).vertex[node1*(*msh).vx_w+l_mctxVXSTRT],
-			     ncomp, feval_result);
+			     0.0, ncomp, feval_result, NULL, NULL);
 	  FUNCTION_FAILURE_HANDLE( err, mesh_func_eval, assem_Dirichlet_BC_tx);
 	  for (j=0; j<ncomp;j++)
 	    {
@@ -2235,7 +2264,7 @@ int assem_Dirichlet_BC_tx(struct projector1 *P, struct mesh *msh,
 	  /* NODE 2 Fct-eval and copying to nodem.V */
 	  err=mesh_func_eval(&(*msh), fu,
 			     &(*msh).vertex[node2*(*msh).vx_w+l_mctxVXSTRT],
-			     ncomp, feval_result);
+			     0.0, ncomp, feval_result, NULL, NULL);
 	  FUNCTION_FAILURE_HANDLE( err, mesh_func_eval, assem_Dirichlet_BC_tx);
 	  for (j=0; j<ncomp;j++)
 	    {
@@ -2251,7 +2280,7 @@ int assem_Dirichlet_BC_tx(struct projector1 *P, struct mesh *msh,
 	      /* NODE m Fct-eval and copying to nodem.V */
 	      err=mesh_func_eval(&(*msh), fu, 
 				 &(*msh).vertex[nodem*(*msh).vx_w+l_mctxVXSTRT],
-				 ncomp, feval_result);
+				 0.0, ncomp, feval_result, NULL, NULL);
 	      FUNCTION_FAILURE_HANDLE( err, mesh_func_eval, assem_Dirichlet_BC_tx);
 	      for (j=0; j<ncomp;j++)
 		{
@@ -2286,7 +2315,7 @@ int assem_Dirichlet_BC_tx(struct projector1 *P, struct mesh *msh,
       {
 	(*P).V[i+s*r]=(*P).V[i]+s*vx_nr;
       }
-  (*P).len=dim*r;
+  (*P).len=ncomp*r;
   
   printf("vx_nr=%d   P.len=%d\n", (int) vx_nr, (int) (*P).len);
   /* end define P */
@@ -2535,13 +2564,14 @@ int max_angle_tx(struct mesh *m, double *maxal, int type
 
 
 /*FUNCTION*/
-int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
-                  double lambda, double mu, FIDX *marker, 
-		  struct solver_settings *set, int type
+int lame_error_est_residual_t2(struct mesh *m, struct vector *uh,
+			       double lambda, double mu, FIDX *marker, 
+			       FIDX *nrmarked, double *globest,
+			       struct solver_settings *set, int type
 /* Estimates the error with a residual-based error estimator for the
    Lame problem.
 
-   Input:  msh         - the mesh
+   Input:  m           - the mesh
 	   uh	       - soltution vector of the discretisized lame problem
            lambda, mu  - Lame constants 
 	   set	       - holding the settings for the process
@@ -2551,6 +2581,9 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 
    Output: marker      - vector of length msh.el_nr with
                          marker[i]==1 -> element i isto be refined,
+           nrmarked    - number of marked elements, i.e. number of
+                         non-zero entries in marker (given by reference)
+           globest     - value of the global estimate (given by reference)
 
    Return: SUCCESS     - success
            FAIL        - failure, see error message, output will not be
@@ -2561,24 +2594,9 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
   FIDX el,bd,i,j,k,l,r,s,eg;
   int err;
 
-  FIDX dim, bas_n, bas_n1d1, dim_bas_n, eg_nr, 
+  FIDX dim, bas_n, dim_bas_n, eg_nr, 
     vx_nr, el_w, vx_w, eg_w, fc_w, bd_w, fu_w, el_nr;
-  dim=2;
-  if ((*m).dim!=dim)
-    {
-      fprintf( stderr,
-	"error_est_lame_residual_t2 is only implemented for 2D meshes!\n");
-      return FAIL;
-    }			    
 
-  if (type!=2)
-    {
-      fprintf( stderr,
-	"error_est_lame_residual_t2 is only implemented fully"
-	" for T2 meshes at the moment!\n");
-      return FAIL;
-    }	
-    
   /* for FORTRAN calls: */
   char   fNoTrans='N';
   double dhelp, done=1.0, dzero=0.0;
@@ -2592,15 +2610,45 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
   double AdetJac;        /* abs(detJac) */
   double *gradu, *hessu; /* first and second derivative of uh */
   
-  double *nk; 		 /* vector containing the squares of the local error 
-			   estimates */
-  double Rh[dim];	 /* residual for an element Rh = f + div.C(eps(uh)) */
-  double rh[dim];	 /* vector for the jump or boundary condition over 
-			   an edge */
+  struct vector nk; 	 /* vector containing the squares of the local error 
+			    estimates */
+  double Rh[3];	 /* residual for an element Rh = f + div.C(eps(uh)) */
+  double rh[3];	 /* vector for the jump or boundary condition over 
+			    an edge */
   double h;		 /* size (=length of an edge or length of split edge)*/
+
+  double jumppoints[3*2]={ 0.5, 0.0,
+		       0.5, 0.5,
+		       0.0,  0.5 };
+  double *jumpp, *jumpgradp, *jumphessp;
+
+  double points[1*2];
+  double *phi, *gradp, *hessp;
+
+  FIDX *el_edge;  
+  FIDX midnode;
+  double n[3];
+
   FIDX l_mctxELNOD1, l_mctxVXSTRT, l_mctxBDTYPE, l_mctxBDEDGE, l_mctxEGNOD1,
 	l_mctxFCEDG1, l_mctxEGCHL1, l_mctxEGBND;
 
+  dim=2;
+  if ((*m).dim!=dim)
+    {
+      fprintf( stderr,
+	"lame_error_est_residual_t2 is only implemented for 2D meshes!\n");
+      return FAIL;
+    }			    
+
+  if (type!=2)
+    {
+      fprintf( stderr,
+	"lame_error_est_residual_t2 is only implemented fully"
+	" for T2 meshes at the moment!\n");
+      return FAIL;
+    }	
+    
+
   switch (type)
     {
     case 1:
@@ -2624,35 +2672,29 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
       l_mctxFCEDG1 = MCT2FCEDG1;
       break;
     default:
-      fprintf(stderr,"error_est_lame_residual_t2: unknown type=%d\n",type);
+      fprintf(stderr,"lame_error_est_residual_t2: unknown type=%d\n",type);
       return FAIL;
     }
 
   /* get integration formula for the jump over the edges, using only midnode */
-  double jumppoints[3*2]={ 0.5, 0.0,
-		       0.5, 0.5,
-		       0.0,  0.5 };
-  double *jumpp, *jumpgradp, *jumphessp;
   if (type==2)
     {
       err= eval_basis( 2, tria, 2, 3, jumppoints,
 		       &bas_n, &jumpp, &jumpgradp, &jumphessp);
       FUNCTION_FAILURE_HANDLE( err, 
-	    eval_basis, error_est_lame_residual_tx);
+	    eval_basis, lame_error_est_residual_t2);
     }
 
 
   /* get integration formula for the residual over an elemtent, using only the midnode */
-  double points[1*2];
-    points[0]=1.0/3.0;
-    points[1]=1.0/3.0;
-  double *phi, *gradp, *hessp;
+  points[0]=1.0/3.0;
+  points[1]=1.0/3.0;
   if (type==2)
     {
       err= eval_basis( 2, tria, 2, 1, points,
 		       &bas_n, &phi, &gradp, &hessp);
       FUNCTION_FAILURE_HANDLE( err, 
-	    eval_basis, error_est_lame_residual_tx);
+	    eval_basis, lame_error_est_residual_t2);
     }
 
   /* for FORTRAN calls */
@@ -2671,23 +2713,25 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
   dim_bas_n = dim*bas_n;
     
   /* allocate memory for gradients, hessian, jacobians,... */    
-  TRY_MALLOC( gradu, dim*dim, double, error_est_lame_residual_tx);
-  TRY_MALLOC( hessu, dim*dim*dim, double, error_est_lame_residual_tx); 
-  TRY_MALLOC( Jac, dim*dim, double, error_est_lame_residual_tx);
-  TRY_MALLOC( Jacinv, dim*dim, double, error_est_lame_residual_tx); 
-  TRY_MALLOC( Jinvgrad, dim_bas_n, double, error_est_lame_residual_tx); 
-  TRY_MALLOC( Jinvhess, dim*dim_bas_n, double, error_est_lame_residual_tx);    
-  TRY_MALLOC( nk, el_nr, double, error_est_lame_residual_tx);
+  TRY_MALLOC( gradu, dim*dim, double, lame_error_est_residual_t2);
+  TRY_MALLOC( hessu, dim*dim*dim, double, lame_error_est_residual_t2); 
+  TRY_MALLOC( Jac, dim*dim, double, lame_error_est_residual_t2);
+  TRY_MALLOC( Jacinv, dim*dim, double, lame_error_est_residual_t2); 
+  TRY_MALLOC( Jinvgrad, dim_bas_n, double, lame_error_est_residual_t2); 
+  TRY_MALLOC( Jinvhess, dim*dim_bas_n, double, lame_error_est_residual_t2);    
+
+  err=vector_alloc( &nk, el_nr);
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, lame_error_est_residual_t2);
+
   for (i=0; i<el_nr; i++) 
     {
       marker[i]=0;		 
-      nk[i] = 0.0;
+      nk.V[i] = 0.0;
     }
   
   /* First: get the elements that belong to an edge 
      (for later jumps [.] over an edge) */  
-  FIDX *el_edge;  
-  TRY_MALLOC( el_edge, 2*eg_nr, FIDX, error_est_lame_residual_tx);
+  TRY_MALLOC( el_edge, 2*eg_nr, FIDX, lame_error_est_residual_t2);
   for (i=0; i<2*eg_nr; i++) el_edge[i] = -1;
   for (el=0; el<el_nr; el++)
     {
@@ -2697,7 +2741,7 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 	  eg=(*m).face[el*fc_w+l_mctxFCEDG1+i];
 	  if ((*m).edge[eg*eg_w+l_mctxEGCHL1]!=-1)
 	    {
-	      printf("edge %d is splitted!\n", j);
+	      printf("edge %d is splitted!\n", (int) eg);
 	      return FAIL;  
 	    }
 	  if (el_edge[2*eg]==-1) el_edge[2*eg]=el;
@@ -2706,7 +2750,7 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 	      if (el_edge[2*eg+1]==-1) el_edge[2*eg+1]=el;
 	      else
 	        {
-		  printf("edge %d belongs to more than two elements!\n", j);
+		  printf("edge %d belongs to more than two elements!\n", (int) eg);
 		  return FAIL;
 		}
 	    }
@@ -2725,10 +2769,8 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 	  bd=(*m).edge[eg*eg_w+l_mctxEGBND];
 	  /* Integrals are approximated as constants, so they are only
 	     evaluated in the midnode */		
-	  FIDX midnode;
 	  midnode = (*m).edge[eg*eg_w + MCT2EGNODM];		
 	  /* get normal vector, no need to scale it */
-	  double n[dim];
 	  n[0]=(*m).vertex[(*m).edge[eg*eg_w + l_mctxEGNOD1+1]
 					*vx_w+l_mctxVXSTRT+1]
 			-(*m).vertex[(*m).edge[eg*eg_w + l_mctxEGNOD1]
@@ -2750,13 +2792,13 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 	      if (eled[0]<0)
 	        {
 		  printf("interior edge %d does not belong to any element!\n"
-			 ,eg);
+			 , (int) eg);
 		  return FAIL;
 		}
 	      if (eled[1]<0)
 	        {
 		  printf("interior edge %d belongs to only one element!\n"
-			 ,eg);
+			 , (int) eg);
 		  return FAIL;
 		}
 		
@@ -2776,7 +2818,8 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 			{
 			  printf("eg %d : midnode %d of the edge does not"
 				 "belong its element!"
-				"(error_est_lame_residual_t2)\n",eg,midnode);
+				"(lame_error_est_residual_t2)\n",
+				 (int) eg, (int) midnode);
 			  return FAIL;
 			} 
 		    }				
@@ -2831,7 +2874,7 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 	      /* rh is done, apply it to nk */
 	      for (s=0;s<2;s++)
 		{	    
-		  nk[eled[s]]+=0.5 * (rh[0]*rh[0]+rh[1]*rh[1]) *h ;
+		  nk.V[eled[s]]+=0.5 * (rh[0]*rh[0]+rh[1]*rh[1]) *h ;
 		}
 	    } /* end of interior edge */
 	  else 
@@ -2850,7 +2893,7 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 		    break;
 		  case 3: /* stressfree Neumann*/
 		    printf("warning: boundary condition %d doesn not need "
-			   "to be used (same as BC 0)\n", cond);			  
+			   "to be used (same as BC 0)\n", (int) cond);			  
 		  case 0: /* no boundary condition, meaning stressfree Neumann*/ 
 		  case 2: /* Neumann with given function g */
 		    if (type==2)
@@ -2869,8 +2912,8 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 			      {
 				printf("eg %d : midnode %d of the edge does"
 				       " not belong its element!"
-				       "(error_est_lame_residual_tx)\n",
-				       eg,midnode);
+				       "(lame_error_est_residual_t2)\n",
+				       (int) eg, (int) midnode);
 				return FAIL;
 			      } 
 			  }
@@ -2893,9 +2936,10 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 						       +MCT2VXSTRT+r];
 				  }
 			        err=mesh_func_eval(m, 
-				      (*m).bound[bd*bd_w+MCT2BDFNCT], x, 2, g );
+					(*m).bound[bd*bd_w+MCT2BDFNCT],
+						   x, 0.0, 2, g, NULL, NULL );
 				FUNCTION_FAILURE_HANDLE( err, mesh_func_eval, 
-				      error_est_lame_residual_tx);
+				      lame_error_est_residual_t2);
 			      }
 		          }
 			  
@@ -2960,17 +3004,18 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 			rh[1]+=dhelp*g[1];
 			      
 			/* rh is done, apply it to nk */
-			nk[el]+=(rh[0]*rh[0]+rh[1]*rh[1]) *h;	      
+			nk.V[el]+=(rh[0]*rh[0]+rh[1]*rh[1]) *h;	      
 		      }
 		    else
 		      {
-			printf("unknown type=%d in error estimator", type);
+			printf("unknown type=%d in error estimator", 
+			       (int) type);
 			return FAIL;
 		      }		  
 		    break;
 		default:
-		  fprintf(stderr,"error_est_lame_residual_tx:"
-			  " unknown bound.type=%d\n",cond);
+		  fprintf(stderr,"lame_error_est_residual_t2:"
+			  " unknown bound.type=%d\n", (int) cond);
 		  return FAIL;
 		}
 	    } /* end of boundary edge */
@@ -2984,8 +3029,8 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
     {    
       /* determine h = length of split edge */
 	{
+	  double n[3];
 	  eg=(*m).face[el*fc_w+MCT2FCEDG1+0];
-	  double n[dim];
 	  n[0]=(*m).vertex[(*m).edge[eg*eg_w + l_mctxEGNOD1+1]
 					*vx_w+l_mctxVXSTRT+1]
 			-(*m).vertex[(*m).edge[eg*eg_w + l_mctxEGNOD1]
@@ -3063,90 +3108,20 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
 			+hessu[1*dim*dim+0*dim+0] )*mu ;
 	  Rh[0] *= dhelp;
 	  Rh[1] *= dhelp;
-	  nk[el]+=(Rh[0]*Rh[0]+Rh[1]*Rh[1]) *h*h;
+	  nk.V[el]+=(Rh[0]*Rh[0]+Rh[1]*Rh[1]) *h*h;
 	}
 
     } /* end loop over elements */
 
   /* Finally, decide which elements are to marked */      
-  /* determine the largest nk*/
-  double nk_max=0.0;
-  double n_global=0.0;
-  for (i=0; i<el_nr; i++) 
-    {
-      if (nk[i]>nk_max) nk_max=nk[i];
-      n_global += nk[i];
-    }  
 
-  /* decide which elements are to be marked */
-  if((*set).adap_mark==0)
-    {  /* get maximum and mark all nk that are a certain 
-	  percentage of it */
-      double tol_nk; 	
-      tol_nk = (*set).adap_mark_par*(*set).adap_mark_par*nk_max;
-      j=0;
-      for (i=0; i<el_nr; i++) 
-        {
-          if (nk[i]>=tol_nk)
-            {      
-	      marker[i]=1;
-	      j++;
-	    }
-        }
-      if (j==0)    
-	{ /* Check if an element has been marked */
-	  printf("Error_estimator:"
-		 " No elements have been marked! nk_max=%f, tol=%f\n",
-		 nk_max, (*set).adap_mark_par);
-	  return FAIL;
-	}
-    }
-  else if ((*set).adap_mark==1)
-    {  /* First sort the list, then mark the largest percentage */
-      int *nk_pos;
-      TRY_MALLOC( nk_pos, el_nr, int, error_est_lame_residual_t2);
-      for (i=0; i<el_nr; i++) nk_pos[i]=i;
-      heapSort(nk, nk_pos, el_nr);
-      j = (int) (((*set).adap_mark_par)*el_nr + 0.5);
-      if (j < 1)
-	{ /* Make sure at least one element is marked */
-	  fprintf(stderr,"Warning in error_est_lame_residual_t2:"
-		  " adap_mark_par=%5.1e too small.\n Marker is set "
-		  "to at least the element with the largest error.\n",
-		  (*set).adap_mark_par );
-	  j=1;
-	}
-      for (i=(el_nr-j); i<el_nr; i++) marker[nk_pos[i]]=1;
-      free(nk_pos);
-    }
-  else if ((*set).adap_mark==2)
-    {  /* First sort the list, then mark the largest until percentage
-	       of global error is marked */
-      int *nk_pos;
-      double global_part, global_goal;
-      TRY_MALLOC( nk_pos, el_nr, int, error_est_lame_residual_tx);
-      for (i=0; i<el_nr; i++) nk_pos[i]=i;
-      heapSort(nk, nk_pos, el_nr);
-      global_goal=(*set).adap_mark_par*n_global;
-      global_part=0.0;
-      for (i=(el_nr-1); ((i>=0)&&(global_part<global_goal)) ; i--)
-	{
-	  marker[nk_pos[i]]=1;
-	  global_part += nk[i];
-	}   
-      free(nk_pos);
-    }
-  else
-    {
-      fprintf(stderr, "error_est_lame_residual_t2: unknown marking type:"
-	     " type=%d\n", (*set).adap_mark);
-      return FAIL;
-    }
+  printf("lame_error_est_residual_t2->");
+  err=gen_error_est_marker(m, &nk, marker, nrmarked, globest, set);
+  FUNCTION_FAILURE_HANDLE( err, gen_error_est_marker,
+			   lame_error_est_residual_t2);
 
-  printf("Error estimator:"
-	 " %d elements marked (=%f). nk_max = %8.3e, n_global = %8.3e\n",
-	 j,((double) j)/((double) el_nr), sqrt(nk_max), sqrt(n_global)   ); 
-     
+  vector_free(&nk);
+  
   free(gradu);
   free(hessu);
   free(Jac);
@@ -3154,7 +3129,6 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
   free(Jinvgrad);
   free(Jinvhess);
   free(el_edge);
-  free(nk);
   free(jumpp);	    
   free(jumpgradp);
   free(jumphessp);
@@ -3165,101 +3139,3 @@ int error_est_lame_residual_t2(struct mesh *m, struct vector *uh,
   return SUCCESS;
 }
 
-
-/*FUNCTION*/
-void heapSort(double *a, int *b, int len
-/* Sorts a vector with the Heap Sort algorithm
-
-   INPUT:        len   - lenght of array a and b
-   
-   INPUT/OUTPUT: a     - array whose elements are to be sorted 
-   
-   OUTPUT:       b     - array holding the new positions of the ordered elements
-
-*/
-){
-
-  int start, end;
-
-  /* start is assigned the index in a of the last parent node */
-  start = (int) ((len - 2) / 2);
-  while (start > -1) 
-    {     
-      /* sift down the node at index start to the proper place such 
-	 that all nodes below the start index are in heap order */
-      siftDown(a, b, start, len-1);
-      start--;
-      /* after sifting down the root all nodes/elements are in heap order */
-    }
-	    
-  end = len - 1;
-  while (end > 0) 
-    {
-      /* swap the root(maximum value) of the heap with the last
-	 element of the heap */
-      swap(a, b, end, 0);
-      /* put the heap back in max-heap order */
-      siftDown(a, b, 0, end-1);
-      /* decrease the size of the heap by one so that the previous 
-	 max value will stay in its proper placement */
-      end--;
-    }
-}
-  
-/*FUNCTION*/
-void siftDown(double *a, int *b, int start, int end
-/* part of the Heap Sort algorithm
-
-  INPUT/OUTPUT: a   - an unordered array a of length count
-		b   - vector holding the new positions of the ordered elements
-
-  INPUT:        end - represents the limit of how far down the heap
-			to sift.
-*/
-){
-  int child, root;
-	
-  root = start;
-  
-  while ( (root*2) < end )
-    {	/* While the root has at least one child */
-      child = root*2+1; /* root*2+1 points to the left child */
-	    
-      /* If the child has a sibling and the child's value is less than its sibling's... */
-      if ( (child < end) && (a[child]<a[child+1]) )
-        {
-	  child++; /* ... then point to the right child instead */
-	}
-      if (a[root]<a[child]) /* out of max-heap order */
-	{
-	  swap(a, b, root, child);
-	  root = child; /* repeat to continue sifting down the child now */
-	}
-      else return;
-    }
-
-}
-  
-/*FUNCTION*/
-void swap(double *a, int *b, int pos1, int pos2
-/* swaps the two entries pos1 and pos2 in the array a and b
-
-  INPUT/OUTPUT : a     - an unordered array a of length count
-                 b     - array holding the new positions 
-		         of the ordered elements
-
-  INPUT:         pos1,
-                 pos2  -positions o be swapped
-*/
-){
-
-  double tempa = a[pos1];;
-  int tempb = b[pos1];
-
-  a[pos1] = a[pos2];
-  b[pos1] = b[pos2];
-
-  a[pos2] = tempa;
-  b[pos2] = tempb;
-
-}
