@@ -411,13 +411,24 @@ struct solver_settings
 			   	   0 - uniform (red) mesh refinement
 			   	   1 - adaptive (beansch green) mesh refinement
 				       with a residual-based error estimator*/
-  FIDX refine_steps;		/* number of refinement steps */
+  FIDX refine_steps;		/* (maximum) number of refinement steps */
+  double refine_stop;		/* error tolerance for the refinement,
+				   stop refinement if the global error
+				   estimate globest<refine_stop */
+  FIDX refine_max_vx;		/* max number vertices, refinement
+				   atempts to stop just before this
+				   vx_nr is exceeded */
+
+
   FIDX adap_mark;		/* Type of marking the element 
 				   (that are to be refined) 
 				   0 - all T with n_T > adap_mark_par* (max n_t)
 				   1 - sort list and mark the
 				       (adap_mark_par) largest */
   double adap_mark_par;	        /* Parameter for the marking */
+
+
+
   FIDX solver;	                /* Type of solver for Ku=f:
 			   	   0 - direct solver
 			     	   1 - PCG with multigrid
@@ -428,7 +439,43 @@ struct solver_settings
   double solver_ini_rtol;	/* relative stop criteria in the solver
 				   for the first solving */
   double solver_ref_rtol;	/* relative stop criteria in the solver
-				   during the refinement (for the interpolated solution) */
+				   during the refinement (for the
+				   interpolated solution) */
+
+  FIDX is_instat;               /* if the problem is instationary, all
+				   the instat_* properties are only
+				   set if is_instat==1
+				   0 - problem is stationary, all
+				       instat_* properties are not
+				       used/set
+				   1 - problem is not stationary */
+  double instat_delta_t0;       /* time step on the initial (coarse)
+                                   mesh, actual time step is computed
+                                   refining the time step according to
+                                   the order of the time integration
+                                   method in comparison to the order
+                                   of the spatial discretisation to
+                                   get balancing of these error
+                                   contributions
+				*/
+  double instat_delta_t_fix;    /* if instat_delta_t_fix>0.0 the fixed
+				   time step size instat_delta_t_fix
+				   is used, and NOT REDUCED during mesh
+				   refinement */
+  double instat_Time0;          /* initial time for the time dependent
+				   solution */
+  double instat_Tend;           /* final time, computation is stopped
+				   if time>instat_Tend */
+  FIDX instat_write_steps;      /* the solution is written every
+				   instat_write_steps time steps
+				   instat_delta_t0 or instat_delta_t_fix,
+				   i.e. if not fixed step size is
+				   used, the number of time steps
+				   before writing the solution
+				   increases with mesh refinement */
+
+
+
   FIDX write_mesh;	        /* What to write to file (optional):
 			           0 - nothing
 				   1 - initial and deformed T2 mesh
@@ -436,6 +483,8 @@ struct solver_settings
 				   3 - initial and refined T1 mesh */
   FIDX write_ssegs;	        /* 1 - Writing svg-file with the
 				       shape parameters (optional) */
+
+
 };
 
 /* end of datastruc.h */
