@@ -22,8 +22,17 @@ void dgbtrs_ ( char *TRANS, int *N, int *KL, int *KU, int *NRHS,
 	       double *AB, int *LDAB, int *IPIV, double *B, int *LDB,
 	       int *INFO );
 
+
+
 /* DGESV( N, NRHS, A, LDA, IPIV, B, LDB, INFO ) */
 void dgesv_( int* N, int* NRHS, double* A, int* LDA, int* IPIV,
 	     double* B, int* LDB, int* INFO );
 
+/* DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
+   LWORK >= (NB+2)*N  */
+void dsyev_( char* JOBZ, char* UPLO, int* N, double* A, int* LDA, 
+	     double* W, double* WORK, int* LWORK, int* INFO );
+
+
+
 #endif
