@@ -202,6 +202,38 @@ int mesh_write_solution_femplot_t1( struct mesh *m, struct vector *x,
                      valid
 */);
 
+int mesh_gnuplot_boundary_circles_t2( FIDX ncirc, double *Rs, double tol,
+				      struct mesh *m,
+				      struct vector *data, FIDX ncomp, int mode,
+				      char *fname_base
+/* assuming that we have ncirc circles around (0,0) as boundary
+   (r=Rs[i]), plot the data over the boundary
+
+   Input:  ncirc     - number of circles
+           Rs        - array of length ncirc, defining the radius of
+                       each circle, the data is written to a separate
+                       file for each circle
+	   tol       - tolerance in testing if a point is on a circle
+	   m         - the mesh that the data corresponds to
+           data      - vector containing the data
+           ncomp     - number of components in the data,  
+           mode      - defines if the components of the data are
+                       written to the file (mode=0), or the norm of
+                       the vector of components associated to each
+                       node of the mesh (mode=1)
+           fname_base- base of the filenames that is written to, the
+                       files will be 
+		         <fname_base>_R1.txt
+		       to
+		         <fname_base>_R<ncirc>.txt
+
+   Output: (the files)
+
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/);
+
 int mesh_write_bound_gnuplot_tx( struct mesh *m, 
 				FIDX namlen, char *name,
 				int type
@@ -266,7 +298,8 @@ int mesh_write_ssegs_svg_tx( struct mesh *m,
 */);
 
 int mesh_write_sseg_svg_file_tx(FILE *datfile, FILE *datfile_mrk,
-				struct mesh *m, FIDX seg, FIDX node1
+				struct mesh *m, FIDX seg, FIDX node1,
+				double diam
 /* writes the segement to the files specified by the file pointers,
    depending on the type and orientation of the segment
 
@@ -280,6 +313,7 @@ int mesh_write_sseg_svg_file_tx(FILE *datfile, FILE *datfile_mrk,
             node1   - first node of the segment, the orientation of
                       the segment is adjusted such that it really has
                       this node as first node
+            diam    - diameter of bounding box, used for scaling the markers
 
    output:  (writes to the file)
 
@@ -287,6 +321,20 @@ int mesh_write_sseg_svg_file_tx(FILE *datfile, FILE *datfile_mrk,
             FAIL    - failure, see error message 
 */);
 
+int mesh_write_sseg_bbox_tx(double *minx, double *maxx, double *miny, double *maxy,
+			    struct mesh *m
+/* find the bounding box for all shape segments in the file
+
+   input:   m       - a t1 or t2 mesh
+            
+   output:  minx,maxx,miny,maxy
+                    - the min and max of all coordinates of shape
+                      segment control points
+
+   return:  SUCCESS - success,
+            FAIL    - failure, see error message 
+*/);
+
 int mesh_read_file_t1( struct mesh *m, struct solver_settings *set, char *name
 /* reads a mesh file, for a descrition of the file format, see the
    example file "mesh_example.f1m"
@@ -788,7 +836,7 @@ int mg_interpolate_t2( struct mgdata *mg,
    
    Input:  lvlin   - the level from which the interpolation shall start
 	   lvlout  - the level to which the interpolation shall take
-	             place, lvlout <= lvlin
+	             place, lvlout >= lvlin
 
 
    In/Out: mg      - multigrid data, the done part is used and therby
@@ -1055,8 +1103,8 @@ int mesh_t2_to_t1( struct mesh *m2, struct mesh *m1
                      valid
 */);
 
-int mesh_func_eval ( struct mesh *m, FIDX tfu, double *point,
-		     FIDX reslen, double *result
+int mesh_func_eval ( struct mesh *m, FIDX tfu, double *point, double time,
+		     FIDX reslen, double *result, double *gradf, double *dotf
 /* evaluates a function as defined by the mesh
 
    Input:  m       - the mesh
@@ -1064,6 +1112,8 @@ int mesh_func_eval ( struct mesh *m, FIDX tfu, double *point,
                      within m.func
 	   point   - point where the function has to be evaluated
 	             (vector of length m.dim)
+           time    - time when the function is to be evaluated,
+                     just provide time=0.0 if stationary problem 
 	   reslen  - length of the provided result vector,
 	             if the function type doesn't match reslen
 	             this function will return FAIL
@@ -1071,7 +1121,14 @@ int mesh_func_eval ( struct mesh *m, FIDX tfu, double *point,
    Output: result  - vector of length reslen (provided by reference),
                      holds the result of the function evaluation on
                      SUCCESSful return
-
+           gradf   - gradient of the function, only modified if
+                     gradf!=NULL, otherwise has to be array of length reslen*(m.dim)
+		     gradf[i*dim+j] is the derivative of the i-th
+		     component of the function wrt. the j-th variable
+		     of space
+	   dotf    - time derivative of the function, only modified if
+	             dotf!=NULL, otherwise has to be vector of length reslen
+                     
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
                      valid
