@@ -37,7 +37,7 @@ HEADER mesh_deform.h
 TO_HEADER:
 */
 
-
+#define _USE_MATH_DEFINES 
 #include <math.h>
 #ifdef fmax
 #warning "math.h"
@@ -56,7 +56,7 @@ TO_HEADER:
 #define __USE_GNU
 #endif
 
-#include <unistd.h>
+//#include <unistd.h>
 #include <string.h>
 
 
