@@ -213,6 +213,33 @@
 				          such that
 				          f(x,y)=c[0]x^2+c[1]y^2+c[2]xy
 					         +c[3]x+c[4]y+c[5]
+ 	                             ==   8.0, 2D Gaussian function
+	                                  (radial only),
+					  FUDAT1 is the begin of a
+				          vector of coefficients c[i]
+				          such that 
+					  f(x,y)=c[2]*exp(-((x-c[0])^2
+					                    +(y-c[1])^2
+							   )/(2*c[3]^2) )
+							   + c[4] 
+ 	                             ==   9.0, 2D Gaussian function
+	                                  (radial only) with center
+	                                  intensity and offset given
+	                                  by other functions,
+					  FUDAT1 is the begin of a
+				          vector of coefficients c[i]
+				          such that 
+					  f(x,y)=f_1*exp(-(
+					                   norm((x,y)-f_0)^2
+					                  )/(2*c[2]^2) )
+							   + f_3
+					  where 
+					  f_0 - vector valued function
+ 					        with id c[0] 
+					  f_1 - scalar valued function
+ 					        with id c[1] 
+					  f_3 - scalar valued function
+ 					        with id c[3] 
 				     ==  10.0, compound function 1 /
  				         distinguished at a hyperplane
 				         FUDAT1 ist the begin of a
@@ -229,6 +256,12 @@
   					       vector type, since
   					       this is done for the
   					       components seperately
+				     ==  31.0, constant in space,
+				               linear in time,
+				          FUDAT1 is the begin of a
+				          vector of coefficients c[i]
+				          such that
+				          f(x,y)=c[1]*t+c[0]
 				  */
 #define MC2XFUDAT1 (MC2XFUTYPE+1) /* begin of function data for first
 				     or only component */
@@ -339,7 +372,7 @@
 				    -1= no BC (but shape segment),
 				     1= Dirichlet (e.g. noslip),
 				     2= Neumann,
-				     3= no stress
+				     3= homogenous Neumann/no stress
 				  */
 #define MCT1BDFNCT (MCT1BDTYPE+1) /* function for boundary condition 
 				     (if applicable), points into the
