@@ -47,9 +47,6 @@ TO_HEADER:
 
 
 
-/* function prototypes */
-#include "navsto_adj.h"
-
 /* prototypes of external functions */
 #include <math.h>
 #include "sparse.h"
@@ -59,6 +56,9 @@ TO_HEADER:
 #include "lin_solver.h"
 #include "navsto_aux.h"
 
+/* function prototypes */
+#include "navsto_adj.h"
+
 /*FUNCTION*/
 int navsto_adj_convert(struct navsto_matrix *K
 /* converts the system matrix of the Navier-Stokes system into its 
@@ -76,7 +76,7 @@ int navsto_adj_convert(struct navsto_matrix *K
            FAIL    - failure, see error message
 */
 		       ){
-  FIDX i, j, d, row, mlvldof;
+  FIDX i, d;
   int  err;
   
   FIDX len, dim, vx_nr;
@@ -88,8 +88,6 @@ int navsto_adj_convert(struct navsto_matrix *K
   struct sparse *oldF, *newF, *newFs;
   struct multilvl *ml;
 
-  double one=1.0;
-
   dim    = (*K).dim;
   vx_nr  = (*K).vx_nr;
   ml     = (*K).mld;
@@ -573,7 +571,9 @@ int navsto_adj_w_precon(void *arg1, struct vector *in,
   /*************************** test *************************
     project the rhs for the Ap solve
    */
-#warning "explain additional projection step in adjoint befroe Ap solve!"  
+#ifdef FEINS_have_warning 
+#warning "explain additional projection step in adjoint befroe Ap solve!"
+#endif
   {
     double sumAprhs=0;
     for (i=0; i<vx_nr; i++)
@@ -681,9 +681,9 @@ int navsto_adj_vel_MG(void *arg1, struct vector *in,
       fprintf(stderr,
 	      "navsto_adj_vel_MG: in or out length doesn't match\n");
       fprintf(stderr,
-	      "should be: %d, in.len=%d, out.len=%d\n", (int) len,
-	      (int) (*in).len, (int) (*out).len);
-      fprintf(stderr, "dim=%d    vx_nr=%d\n", (int) dim, (int) vx_nr);
+	      "should be: %"dFIDX", in.len=%"dFIDX", out.len=%"dFIDX"\n",
+	      len, (*in).len,  (*out).len);
+      fprintf(stderr, "dim=%"dFIDX"    vx_nr=%"dFIDX"\n",  dim,  vx_nr);
       return FAIL;
     }
 
@@ -1454,7 +1454,9 @@ int navsto_Psi_dRdF_t21(struct vector *Psi_dRdF,
 	      sumw+=weight * fabs(detJac);
 	    }
 
-#warning navsto_Psi_dRdF_t21: no rhs assembly yet!
+#ifdef FEINS_have_warning 
+#warning "navsto_Psi_dRdF_t21: no rhs assembly yet!"
+#endif
 	} /* end loop over all integration points */
       
       /***************************************************************
@@ -1890,7 +1892,7 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 	default:
 	  fprintf(stderr,
 		  "navsto_dIdX_t21: unknown pcsurf pccrit type!"
-		  "(pccrit %d)\n", (int) pccrit);
+		  "(pccrit %"dFIDX")\n",  pccrit);
 	  return FAIL;
 	}
 
@@ -2239,8 +2241,8 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 		      break;
 		    default:
 		      fprintf(stderr, "navsto_dIdX_t21: "
-			      "unknown pcsurf pccrit type!(pccrit %d)\n",
-			      (int) pccrit);
+			      "unknown pcsurf pccrit type!(pccrit %"dFIDX")\n",
+			       pccrit);
 		      return FAIL;
 		    }
 		} /* end loop over all integration points */
@@ -2567,8 +2569,8 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 		    else
 		      {
 			fprintf(stderr,"navsto_dIdX_t21: pccrit type 4,"
-			      " invalid component!(pccrit %d)\n", 
-				(int) pccrit);
+			      " invalid component!(pccrit %"dFIDX")\n", 
+				 pccrit);
 			return FAIL;
 		      }
 
@@ -2634,8 +2636,8 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 		    else
 		      {
 			fprintf(stderr,"navsto_dIdX_t21: pccrit type 4,"
-			      " invalid component!(pccrit %d)\n",
-				(int) pccrit);
+			      " invalid component!(pccrit %"dFIDX")\n",
+				 pccrit);
 			return FAIL;
 		      }
 		  } /* end if tloop==1 */
@@ -2655,8 +2657,8 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 		    else
 		      {
 			fprintf(stderr,"navsto_dIdX_t21: pccrit type 5,"
-			      " invalid component!(pccrit %d)\n", 
-				(int) pccrit);
+			      " invalid component!(pccrit %"dFIDX")\n", 
+				 pccrit);
 			return FAIL;
 		      }
 
@@ -2727,8 +2729,8 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 		    else
 		      {
 			fprintf(stderr,"navsto_dIdX_t21: pccrit type 5,"
-			      " invalid component!(pccrit %d)\n", 
-				(int) pccrit);
+			      " invalid component!(pccrit %"dFIDX")\n", 
+				 pccrit);
 			return FAIL;
 		      }
 		  } /* end if tloop==1 */
@@ -2781,8 +2783,8 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 			else
 			  {
 			    fprintf(stderr,"navsto_dIdX_t21: pccrit type 6,"
-				    " invalid component!(pccrit %d)\n", 
-				    (int) pccrit);
+				    " invalid component!(pccrit %"dFIDX")\n", 
+				     pccrit);
 			    return FAIL;
 			  }
 		      } /* end loop over components */
@@ -2791,7 +2793,7 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 	      default:
 		fprintf(stderr,
 			"navsto_dIdX_t21: unknown pcvol pccrit type!"
-			"(pccrit %d)\n", (int) pccrit);
+			"(pccrit %"dFIDX")\n",  pccrit);
 		return FAIL;
 	      }
 	  } /* end loop over all integration points */
@@ -2850,7 +2852,7 @@ int navsto_dIdX_t21( struct mesh *msh, struct navsto_matrix *K,
 	default:
 	  fprintf(stderr,
 		  "navsto_dIdX_t21: unknown pcvol pccrit type!"
-		  "(pccrit %d), final corrections\n", (int) pccrit);
+		  "(pccrit %"dFIDX"), final corrections\n",  pccrit);
 	  return FAIL;
 	}
     }
