@@ -47,9 +47,6 @@ TO_HEADER:
 
 
 
-/* function prototypes */
-#include "navsto_aux.h"
-
 /* prototypes of external functions */
 #include <math.h>
 #include <string.h>
@@ -61,6 +58,9 @@ TO_HEADER:
 #include "stokes_aux.h"
 #include "navstoassem.h"
 
+/* function prototypes */
+#include "navsto_aux.h"
+
 /*FUNCTION*/
 int navsto_write_line_u_t2( struct mesh *m, struct vector *usol,
 			    int xy, double dxy,
@@ -109,8 +109,9 @@ int navsto_write_line_u_t2( struct mesh *m, struct vector *usol,
   if ((*usol).len<2*(*m).vx_nr)
     {
       fprintf(stderr,
-	      "navsto_write_line_u_t2: usol.len=%d doesn't fit vx_nr=%d !\n",
-	      (int) (*usol).len, (int) (*m).vx_nr);
+	      "navsto_write_line_u_t2: usol.len=%"dFIDX
+	      " doesn't fit vx_nr=%"dFIDX" !\n",
+	       (*usol).len,  (*m).vx_nr);
       return FAIL;
     }
 
@@ -1075,7 +1076,7 @@ int navsto_matrices_write_files(struct navsto_matrix *K, char *basename
 	{
 	  /* they might be unordered, so make sure we store the index
 	     as well (matlab counts indices from 1, we from 0) */
-	  fprintf(out, " %7d   %+24.16e\n", (int) (pdof[i]+1),
+	  fprintf(out, " %7"dFIDX"   %+24.16e\n",  (pdof[i]+1),
 		  (*K).weight[i]);
 	}
     }
@@ -1101,7 +1102,7 @@ int navsto_matrices_write_files(struct navsto_matrix *K, char *basename
   for (i=0; i<(*K).bn_nr; i++)
     {
       /* (matlab counts indices from 1, we from 0) */
-      fprintf(out, " %7d\n", (int) ((*K).nodes[i]+1) );
+      fprintf(out, " %7"dFIDX"\n",  ((*K).nodes[i]+1) );
     }
 
   /* close the file */
@@ -1887,7 +1888,7 @@ int navsto_projector_vel_MG(void *arg1, struct vector *in,
 		  else
 		    alpha=1.0;
 		  
-		  printf("\nlvl=%d   alpha=%e", (int) lvl, alpha);
+		  printf("\nlvl=%d   alpha=%e",  lvl, alpha);
 		  /* negative alphas are not to be trusted, thus replaced
 		     by alpha=1*/
 		  if (alpha<0.25) alpha=1.0;
@@ -2229,7 +2230,7 @@ int navsto_projector_vel_W_CGC(struct navsto_matrix *K,
   if (( lvl>lmax ) || ( lvl<lvlmin ))
     {
       fprintf(stderr, "navsto_projector_vel_W_CGC: "
-	      "lvl out of range! lvl=%d\n", (int) lvl);
+	      "lvl out of range! lvl=%"dFIDX"\n",  lvl);
       return FAIL;
     }
 
@@ -2533,7 +2534,7 @@ int navsto_precon_p_lapl_bpx(void *notused, struct vector *in,
   for (i=hi_end; i>=hi_start; i--)
     {
       level=(*m).hier[i*hi_w+MCT2HILVL];
-      if (level<=0) printf("error, level=%d\n", (int) level);
+      if (level<=0) printf("error, level=%"dFIDX"\n",  level);
       if (level_old!=level)
 	{
 	  /* initialise the new (coarser) level */
@@ -2952,7 +2953,7 @@ int navsto_mg_Wcycle_SDFEM(struct navsto_matrix *K, struct vector *bh,
     double normres=0.0;
     for (i=0; i<bigN; i++)
       normres+= (*bh).V[i] * (*bh).V[i];
-    printf("W-cycle in   lvl=%d   norm(b)=%e\n",(int) lvl,sqrt(normres));
+    printf("W-cycle in   lvl=%d   norm(b)=%e\n", lvl,sqrt(normres));
   }
 #endif
 
@@ -2984,7 +2985,7 @@ int navsto_mg_Wcycle_SDFEM(struct navsto_matrix *K, struct vector *bh,
 	for (i=0; i<bigN; i++)
 	  normres+= (*rh).V[i] * (*rh).V[i];
 
-	printf(" pre smooth  lvl=%d   norm(r)=%e\n",(int) lvl,sqrt(normres));
+	printf(" pre smooth  lvl=%d   norm(r)=%e\n", lvl,sqrt(normres));
       }
 #endif
 
@@ -3086,7 +3087,7 @@ int navsto_mg_Wcycle_SDFEM(struct navsto_matrix *K, struct vector *bh,
 	for (i=0; i<bigN; i++)
 	  normres+= (*rh).V[i] * (*rh).V[i];
 
-	printf(" post smooth lvl=%d   norm(r)=%e\n",(int) lvl,sqrt(normres));
+	printf(" post smooth lvl=%d   norm(r)=%e\n", lvl,sqrt(normres));
       }
 #endif
 
@@ -3139,7 +3140,7 @@ int navsto_mg_Wcycle_SDFEM(struct navsto_matrix *K, struct vector *bh,
     for (i=0; i<bigN; i++)
       normres+= (*rh).V[i] * (*rh).V[i];
 
-    printf("W-cycle out  lvl=%d   norm(r)=%e\n",(int) lvl,sqrt(normres));
+    printf("W-cycle out  lvl=%d   norm(r)=%e\n", lvl,sqrt(normres));
   }
 #endif
 
@@ -3445,7 +3446,7 @@ int navsto_mg_smoother_SDFEM(struct vector *x, struct vector *rhs,
 	  }
       }
       /* local solution update is now in lrhsv */
-      if (ndofs==1) printf("me has ndofs==1: thep=%d\n", (int) thep);
+      if (ndofs==1) printf("me has ndofs==1: thep=%"dFIDX"\n",  thep);
 
       /* apply the weighting on the velocity dofs of lrhs */
       for (i=0; i<ndofs; i++)
