@@ -58,13 +58,9 @@ TO_HEADER:
 #include "assembly.h"
 #include "mesh_deform.h"
 
-#include <unistd.h>
+//#include <unistd.h>
 #include <string.h>
 
-  /* for gettimeofday */
-#include <sys/types.h>
-#include <sys/time.h>
-
 
 #define VERBOSE 4
 
@@ -127,7 +123,7 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
   FILE *mark;
 
   char solver[15];
-  FIDX dim, vx_nr, i, j, d, step, linmeth, nu_chng;
+  FIDX dim, vx_nr, i, d, step, linmeth, nu_chng;
   int  iter, err;
 
   FIDX lvl, lvl_min;
@@ -275,8 +271,8 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 		for (d=0; d<dim; d++)
 		  {
 		    if (x_old.V[K.nodes[i]+d*vx_nr]!=x.V[K.nodes[i]+d*vx_nr])
-		      printf("xref node %d, dim %d is strange\n",
-			     (int) K.nodes[i], (int) d);
+		      printf("xref node %"dFIDX", dim %"dFIDX" is strange\n",
+			      K.nodes[i],  d);
 		    x_old.V[K.nodes[i]+d*vx_nr]=x.V[K.nodes[i]+d*vx_nr];
 		  }
 
@@ -464,10 +460,10 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 	      ||((VERBOSE>1)
 		  &&( (deltax<=nleps)&&(resi<=nleps) ) ))
 	    {
-	      printf("nu %8.1e st %3d  %8s: N=%6d, it=%4d, "
+	      printf("nu %8.1e st %3"dFIDX"  %8s: N=%6"dFIDX", it=%4d, "
 		     "i_it: %4d %4d %4d %4d err=%2d, |res_l|=%12.6e, "
 		     "|dx|=%9.3e, ",
-		     nu_akt, (int) step, solver, (int) x.len, iter,
+		     nu_akt,  step, solver,  x.len, iter,
 		     K.innercount[0], K.innercount[1],
 		     K.innercount[2], K.innercount[3],
 		     err, resi, deltax); /* */
@@ -526,16 +522,18 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 
 	      /* write the solution values for comparison */
 	      {
+		char fname[300];
+
 		soli.V     = &sol.V[0];
 		soli.len   = (dim)*vx_nr;
 		soli.n_max = (dim)*vx_nr;
-		char fname[300];
-		sprintf(fname, "visual/line.x.%d.txt", (int) lvl);
+
+		sprintf(fname, "visual/line.x.%"dFIDX".txt",  lvl);
 		err=navsto_write_line_u_t2( &msh2, &soli, 0, 0.5, fname );
 		FUNCTION_FAILURE_HANDLE( err,
 					 mesh_write_line_u_t2,
 					 navsto_solver); 
-		sprintf(fname, "visual/line.y.%d.txt", (int) lvl);
+		sprintf(fname, "visual/line.y.%"dFIDX".txt",  lvl);
 		err=navsto_write_line_u_t2( &msh2, &soli, 1, 0.5,
 					    fname ); 
 		FUNCTION_FAILURE_HANDLE( err,
@@ -619,7 +617,7 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 	      printf("\nperformance criteria eval: err=%d\n",err);
 	      for (i=0; i<msh2.pc_nr; i++)
 		{
-		  printf("crit %d: %e\n", (int) i, pccrit->V[i]);
+		  printf("crit %"dFIDX": %e\n",  i, pccrit->V[i]);
 		}
 	    }
 	}
