@@ -58,14 +58,9 @@ TO_HEADER:
 #include "assembly.h"
 #include "mesh_deform.h"
 
-#include <unistd.h>
+//#include <unistd.h>
 #include <string.h>
 
-  /* for gettimeofday */
-#include <sys/types.h>
-#include <sys/time.h>
-
-
 #define VERBOSE 4
 
 #define TIMEGET {\
@@ -119,7 +114,7 @@ int navsto_solver_bcontrol(struct mesh *msh1, FIDX lvl_max,
   FILE *mark;
 
   char solver[15];
-  FIDX dim, vx_nr, i, j, d, step, linmeth, nu_chng;
+  FIDX dim, vx_nr, i, j, step, linmeth, nu_chng;
   int  iter, err;
 
   FIDX lvl, lvl_min;
@@ -402,10 +397,10 @@ int navsto_solver_bcontrol(struct mesh *msh1, FIDX lvl_max,
 	      ||((VERBOSE>1)
 		  &&( (deltax<=nleps)&&(resi<=nleps) ) ))
 	    {
-	      printf("nu %8.1e st %3d  %8s: N=%6d, it=%4d, "
+	      printf("nu %8.1e st %3"dFIDX"  %8s: N=%6"dFIDX", it=%4d, "
 		     "i_it: %4d %4d %4d %4d err=%2d, |res_l|=%12.6e, "
 		     "|dx|=%9.3e, ",
-		     nu_akt, (int) step, solver, (int) x.len, iter,
+		     nu_akt,  step, solver,  x.len, iter,
 		     K.innercount[0], K.innercount[1],
 		     K.innercount[2], K.innercount[3],
 		     err, resi, deltax); /* */
@@ -467,12 +462,12 @@ int navsto_solver_bcontrol(struct mesh *msh1, FIDX lvl_max,
 	      /* write the solution values for comparison */
 	      {
 		char fname[300];
-		sprintf(fname, "visual/line.x.%d.txt", (int) lvl);
+		sprintf(fname, "visual/line.x.%"dFIDX".txt",  lvl);
 		err=navsto_write_line_u_t2( &msh2, &soli, 0, 0.5, fname );
 		FUNCTION_FAILURE_HANDLE( err,
 					 mesh_write_line_u_t2,
 					 navsto_solver_bcontrol); 
-		sprintf(fname, "visual/line.y.%d.txt", (int) lvl);
+		sprintf(fname, "visual/line.y.%"dFIDX".txt",  lvl);
 		err=navsto_write_line_u_t2( &msh2, &soli, 1, 0.5,
 					    fname ); 
 		FUNCTION_FAILURE_HANDLE( err,
@@ -580,7 +575,7 @@ int navsto_solver_bcontrol(struct mesh *msh1, FIDX lvl_max,
 	      printf("\nperformance criteria eval: err=%d\n",err);
 	      for (i=0; i<msh2.pc_nr; i++)
 		{
-		  printf("crit %d: %e\n", (int) i, pccrit->V[i]);
+		  printf("crit %"dFIDX": %e\n",  i, pccrit->V[i]);
 		}
 	    }
 	}
@@ -609,6 +604,9 @@ int navsto_solver_bcontrol(struct mesh *msh1, FIDX lvl_max,
       FIDX tc;
       struct vector *DIDuB, *dIdu, *dIduI, *dIduB;
       struct vector Psi, y;
+	  double norm_dIduI, norm_adjres;
+	  FIDX icount;
+	  FIDX *fctn_id2bc_sections;
       
       FIDX bc_sections;
 
@@ -690,7 +688,6 @@ int navsto_solver_bcontrol(struct mesh *msh1, FIDX lvl_max,
 		}	      
 	    }
 
-	  double norm_dIduI, norm_adjres;
 	  /* compute the norm of the adjoint rhs */
 	  normy=0.0;
 	  for (i=0; i<dIduI[tc].len; i++)
@@ -907,8 +904,6 @@ int navsto_solver_bcontrol(struct mesh *msh1, FIDX lvl_max,
 	      DIDB[tc].V[i]=0.0;
 	    }
 
-	  FIDX icount;
-	  FIDX *fctn_id2bc_sections;
 
 	  TRY_MALLOC(fctn_id2bc_sections,msh2.fu_nr, FIDX, navsto_solver_bcontrol);
 
