@@ -45,15 +45,13 @@ TO_HEADER:
 */
 
 
+/* prototypes of external functions */
+#include <math.h>
+#include "feins_lapack.h"
 
 /* function prototypes */
 #include "sparse.h"
 
-/* prototypes of external functions */
-#include <math.h>
-
-#include "feins_lapack.h"
-
 /*FUNCTION*/
 int sparse_flex_alloc(struct sparse *K, FIDX nrows
 /* creates a sparse matrix in the initial flexible format with nrows rows 
@@ -93,7 +91,7 @@ int sparse_flex_alloc(struct sparse *K, FIDX nrows
 }
 
 /*FUNCTION*/
-int sparse_alloc(struct sparse *K, FIDX nrows, FIDX density
+int sparse_alloc(struct sparse *K, FIDX nrows, FIDX notused_density
 /* OBSOLETE, kept for ease of switching to new matrix format;
    creates a sparse matrix in the initial flexible format with nrows rows 
    
@@ -229,7 +227,7 @@ int sparse_row_empty(struct sparse *K, FIDX row
 
   if ((row<0)||(row>=(*K).row_nr))
     {
-      fprintf(stderr, "sparse_row_empty: row %d invalid !\n", (int) row);
+      fprintf(stderr, "sparse_row_empty: row %"dFIDX" invalid !\n",  row);
       return FAIL;
     }
 
@@ -298,8 +296,8 @@ int sparse_mat_write_file(struct sparse *A, char *name
 	  curr=&(*A).flex_rows[i];
 	  while (curr != NULL)
 	    {
-	      fprintf(out, "  %7d  %7d   %+24.16e\n",
-		      (int) (i+1), (int) ((curr->ix)+1),
+	      fprintf(out, "  %7"dFIDX"  %7"dFIDX"   %+24.16e\n",
+		       (i+1),  ((curr->ix)+1),
 		      curr->dx); /* matlab counts indices
 				    from 1, we from 0 */
 	      /* next collumn */
@@ -315,9 +313,9 @@ int sparse_mat_write_file(struct sparse *A, char *name
 	  m=(*A).rows[i+1];
 	  for (j=(*A).rows[i]; j<m; j++)
 	    {
-	      /* fprintf(out, "%7d %7d %+16.8e\n", /* for smaller files*/
-	      fprintf(out, "  %7d  %7d   %+24.16e\n",
-		      (int) (i+1), (int) ( (*A).cols[j] + 1 ),
+	      /* fprintf(out, "%7"dFIDX" %7"dFIDX" %+16.8e\n", /* for smaller files*/
+	      fprintf(out, "  %7"dFIDX"  %7"dFIDX"   %+24.16e\n",
+		       (i+1),  ( (*A).cols[j] + 1 ),
 		      (*A).A[j]); /* matlab counts indices
 				     from 1, we from 0 */
 	    }
@@ -351,7 +349,8 @@ int sparse_mat_read_file(struct sparse *A, char *name
 */
 			  ){
   FIDX   i, j, row_nr;
-  int    int_i, int_j, happy, err;
+  FIDX   int_i, int_j; 
+  int    happy, err;
   double value;
 
   FILE *in;
@@ -371,7 +370,7 @@ int sparse_mat_read_file(struct sparse *A, char *name
   while (happy!=0)
     {
       int nr;
-      nr = fscanf(in, "%d  %d   %le\n", &int_i, &int_j, &value);
+      nr = fscanf(in, "%"dFIDX"  %"dFIDX"   %le\n", &int_i, &int_j, &value);
       if (nr==3)
 	{
 	  if (int_i>row_nr) row_nr=int_i;
@@ -394,7 +393,7 @@ int sparse_mat_read_file(struct sparse *A, char *name
     {
       int nr;
       double *entry;
-      nr = fscanf(in, "%d  %d   %le\n", &int_i, &int_j, &value);
+      nr = fscanf(in, "%"dFIDX"  %"dFIDX"   %le\n", &int_i, &int_j, &value);
       if (nr==3)
 	{
 	  /* matlab counts indices from 1, we from 0 */
@@ -405,7 +404,7 @@ int sparse_mat_read_file(struct sparse *A, char *name
 	    {
 	      fprintf(stderr, "sparse_mat_read_file: "
 		      "coulnd get/create entry in K for "
-		      "row=%d col=%d ", (int) i, (int) j);
+		      "row=%"dFIDX" col=%"dFIDX" ",  i,  j);
 	      fclose(in);
 	      return FAIL;
 	    }
@@ -443,7 +442,7 @@ int sparse_mat_transp(struct sparse *A, struct sparse *B
 */
 		     ){
   int  err;
-  FIDX i, j, endrow, col, row_nr;
+  FIDX i, j, endrow, row_nr;
   double *entry;
 
   row_nr=(*A).row_nr;
@@ -521,20 +520,17 @@ int sparse_mul_mat_vec(void *arg1, struct vector *vec,
            FAIL    - failure, see error message
 */
 		     ){
-  FIDX i, j, endrow, col, col_max, row_max;
-
-  /* simpliefied acces to A, input vector iv and output ov */
-  double *A, *iv, *ov;
-  double axrow;
-  FIDX   *Arows, *Acols;
-
-  int err;
+  FIDX col_max, row_max;
 
   struct sparse *K;
   K=arg1;
 
+  row_max = (*K).row_nr;
+  col_max = (*vec).len;
+
   if ( (*K).type == SP_TYPE_FLEX )
     {
+      int err;
      /* fprintf( stderr, "WARNING: sparse_mul_mat_vec, \n"
 	       "coverting matrix from flexible to compressed "
 	       "row format \n\n"); */
@@ -549,8 +545,25 @@ int sparse_mul_mat_vec(void *arg1, struct vector *vec,
       return FAIL;
     }
 
-  row_max = (*K).row_nr;
-  col_max = (*vec).len;
+  if (row_max != (*out).len)
+    {
+      fprintf(stderr, "sparse_mul_mat_vec: "
+	      "supplied vector out has wrong size\n");
+      fprintf(stderr, "row_max=%"dFIDX", out.len=%"dFIDX"\n",
+	       row_max,  (*out).len);
+      return FAIL;
+    }
+
+#pragma omp parallel
+  {
+  FIDX i, j, endrow, col;
+
+  /* simpliefied acces to A, input vector iv and output ov */
+  double *A, *iv, *ov;
+  double axrow;
+  FIDX   *Arows, *Acols;
+
+  int err;
 
   A       = (*K).A;
   Arows   = (*K).rows;
@@ -558,16 +571,9 @@ int sparse_mul_mat_vec(void *arg1, struct vector *vec,
   iv      = (*vec).V;
   ov      = (*out).V;
 
-  if (row_max != (*out).len)
-    {
-      fprintf(stderr, "sparse_mul_mat_vec: "
-	      "supplied vector out has wrong size\n");
-      fprintf(stderr, "row_max=%d, out.len=%d\n",
-	      (int) row_max, (int) (*out).len);
-      return FAIL;
-    }
 
   /* loop over all rows */
+#pragma omp for
   for (i=0; i<row_max; i++)
     {
       axrow=0.0;
@@ -583,7 +589,7 @@ int sparse_mul_mat_vec(void *arg1, struct vector *vec,
 	    {
 	      fprintf(stderr,
 		      "sparse_mul_mat_vec: invalid col>=col_max\n");
-	      return FAIL;
+	      FEINS_FAILURE_ACTION;
 	    }
 #endif
 
@@ -593,6 +599,7 @@ int sparse_mul_mat_vec(void *arg1, struct vector *vec,
       ov[i]=axrow;
       /* next row */
     }
+  } /* end omp parallel */
 
   return SUCCESS;
 }
@@ -619,16 +626,9 @@ int sparse_GS_sweep_fwd(struct sparse *K, struct vector *b,
            FAIL    - failure, see error message
 */
 		     ){
-  FIDX i, j, endrow, sweep, col, col_max, row_max;
-  int err;
-
-  /* simpliefied acces to A, b, x and invdiag */
-  double *A, *bv, *xv, *invdiagV;
-  double axrow;
-  FIDX   *Arows, *Acols;
-
   if ( (*K).type == SP_TYPE_FLEX )
     {
+      int err;
       /* fprintf( stderr, "WARNING: sparse_GS_sweep_fwd, \n"
 	       "coverting matrix from flexible to compressed "
 	       "row format \n\n"); */
@@ -643,6 +643,29 @@ int sparse_GS_sweep_fwd(struct sparse *K, struct vector *b,
       return FAIL;
     }
 
+  if ((((*K).row_nr != (*x).len)||(((*K).row_nr != (*b).len)))
+      ||((*K).row_nr != (*invdiag).len))
+    {
+      fprintf(stderr, "sparse_GS_sweep_fwd: "
+	      "supplied vector has wrong size\n");
+      fprintf(stderr, "K.(*K).row_nr=%"dFIDX", b.len=%"dFIDX", invdiag.len=%"dFIDX", x.len=%"dFIDX"\n",
+	       (*K).row_nr,  (*b).len, 
+	       (*invdiag).len,  (*x).len);
+      return FAIL;
+    }
+
+  // #pragma omp parallel
+  { 
+
+  FIDX i, j, endrow, sweep, col, col_max, row_max;
+  int err;
+
+  /* simpliefied acces to A, b, x and invdiag */
+  double *A, *bv, *xv, *invdiagV;
+  double axrow;
+  FIDX   *Arows, *Acols;
+
+
 
   row_max = (*K).row_nr;
   col_max = (*x).len;
@@ -654,20 +677,11 @@ int sparse_GS_sweep_fwd(struct sparse *K, struct vector *b,
   xv      = (*x).V;
   invdiagV= (*invdiag).V;
 
-  if (((row_max != (*x).len)||((row_max != (*b).len)))
-      ||(row_max != (*invdiag).len))
-    {
-      fprintf(stderr, "sparse_GS_sweep_fwd: "
-	      "supplied vector has wrong size\n");
-      fprintf(stderr, "K.row_max=%d, b.len=%d, invdiag.len=%d, x.len=%d\n",
-	      (int) row_max, (int) (*b).len, 
-	      (int) (*invdiag).len, (int) (*x).len);
-      return FAIL;
-    }
 
   for (sweep=0; sweep<sweeps; sweep++)
     {
       /* loop over all rows */
+      //#pragma omp for ordered
       for (i=0; i<row_max; i++)
 	{
 	  
@@ -684,7 +698,7 @@ int sparse_GS_sweep_fwd(struct sparse *K, struct vector *b,
 		{
 		  fprintf(stderr,
 			  "sparse_GS_sweep_fwd: invalid col>=col_max\n");
-		  return FAIL;
+		  FEINS_FAILURE_ACTION;
 		}
 #endif
 
@@ -694,10 +708,14 @@ int sparse_GS_sweep_fwd(struct sparse *K, struct vector *b,
 
 	  /* multiplication of this row complete, apply GS correction
 	     to x */
+	  //#pragma omp ordered
 	  xv[i] -= (axrow - bv[i]) * invdiagV[i];
-
 	} /* next row */
+
+	
     } /* next sweep */
+
+ } /* end omp parallel */
   return SUCCESS;
 }
 
@@ -760,9 +778,9 @@ int sparse_GS_sweep_bwd(struct sparse *K, struct vector *b,
     {
       fprintf(stderr, "sparse_GS_sweep_bwd: "
 	      "supplied vector has wrong size\n");
-      fprintf(stderr, "K.row_max=%d, b.len=%d, invdiag.len=%d, x.len=%d\n",
-	      (int) row_max, (int) (*b).len,
-	      (int) (*invdiag).len, (int) (*x).len);
+      fprintf(stderr, "K.row_max=%"dFIDX", b.len=%"dFIDX", invdiag.len=%"dFIDX", x.len=%"dFIDX"\n",
+	       row_max,  (*b).len,
+	       (*invdiag).len,  (*x).len);
       return FAIL;
     }
 
@@ -872,9 +890,9 @@ int sparse_GS_sweep_sorted(struct sparse *K, struct vector *b,
     {
       fprintf(stderr, "sparse_GS_sweep_sorted: "
 	      "supplied vector has wrong size\n");
-      fprintf(stderr, "K.row_max=%d, b.len=%d, invdiag.len=%d, x.len=%d\n",
-	      (int) row_max, (int) (*b).len,
-	      (int) (*invdiag).len, (int) (*x).len);
+      fprintf(stderr, "K.row_max=%"dFIDX", b.len=%"dFIDX", invdiag.len=%"dFIDX", x.len=%"dFIDX"\n",
+	       row_max,  (*b).len,
+	       (*invdiag).len,  (*x).len);
       return FAIL;
     }
 
@@ -989,8 +1007,8 @@ int sparse_vec_mat_vec(struct sparse *K, struct vector *u,
     {
       fprintf(stderr, "sparse_vec_mat_vec: "
 	      "supplied vector v has wrong size\n");
-      fprintf(stderr, "row_max=%d, v.len=%d\n",
-	      (int) row_max, (int) (*v).len);
+      fprintf(stderr, "row_max=%"dFIDX", v.len=%"dFIDX"\n",
+	       row_max,  (*v).len);
       return FAIL;
     }
 
@@ -1044,18 +1062,17 @@ int sparse_mul_mat_vec_add(struct sparse *K, struct vector *vec,
            FAIL    - failure, see error message
 */
 		     ){
-  FIDX i, j, endrow, col, col_max, row_max;
-  int err;
+  FIDX col_max, row_max;
 
-  double *A, *iv, *ov;
-  double axrow;
-  FIDX   *Arows, *Acols;
+  row_max = (*K).row_nr;
+  col_max = (*vec).len;
 
   if ( (*K).type == SP_TYPE_FLEX )
     {
-      fprintf( stderr, "WARNING: sparse_mul_mat_vec_add, \n"
+      int err;
+     /* fprintf( stderr, "WARNING: sparse_mul_mat_vec_add, \n"
 	       "coverting matrix from flexible to compressed "
-	       "row format \n\n");
+	       "row format \n\n"); */
       err = sparse_convert_compressed_row( K );
       FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
 			       sparse_mul_mat_vec_add );
@@ -1067,9 +1084,25 @@ int sparse_mul_mat_vec_add(struct sparse *K, struct vector *vec,
       return FAIL;
     }
 
+  if (row_max != (*out).len)
+    {
+      fprintf(stderr, "sparse_mul_mat_vec_add: "
+	      "supplied vector out has wrong size\n");
+      fprintf(stderr, "row_max=%"dFIDX", out.len=%"dFIDX"\n",
+	       row_max,  (*out).len);
+      return FAIL;
+    }
 
-  row_max=(*K).row_nr;
-  col_max=(*vec).len;
+#pragma omp parallel
+  {
+  FIDX i, j, endrow, col;
+
+  /* simpliefied acces to A, input vector iv and output ov */
+  double *A, *iv, *ov;
+  double axrow;
+  FIDX   *Arows, *Acols;
+
+  int err;
 
   A       = (*K).A;
   Arows   = (*K).rows;
@@ -1078,14 +1111,8 @@ int sparse_mul_mat_vec_add(struct sparse *K, struct vector *vec,
   ov      = (*out).V;
 
 
-  if (row_max != (*out).len)
-    {
-      fprintf(stderr, "sparse_mul_mat_vec_add: "
-	      "supplied vector out has wrong size\n");
-      return FAIL;
-    }
-
   /* loop over all rows */
+#pragma omp for
   for (i=0; i<row_max; i++)
     {
       axrow=0.0;
@@ -1100,7 +1127,7 @@ int sparse_mul_mat_vec_add(struct sparse *K, struct vector *vec,
 	    {
 	      fprintf(stderr,
 		      "sparse_mul_mat_vec_add: invalid col>=col_max\n");
-	      return FAIL;
+	      FEINS_FAILURE_ACTION;
 	    }
 #endif
 
@@ -1110,6 +1137,8 @@ int sparse_mul_mat_vec_add(struct sparse *K, struct vector *vec,
 	}
       ov[i] += axrow;
     } /* next row */
+  } /* end omp parallel */
+
   return SUCCESS;
 }
 
@@ -1172,8 +1201,8 @@ int sparse_row_tim_vec(struct sparse *K, FIDX rown, FIDX *rows,
     {
       fprintf(stderr, "sparse_row_tim_vec: "
 	      "supplied vector out has wrong size\n");
-      fprintf(stderr, "rown=%d, out.len=%d\n",
-	      (int) rown, (int) (*out).len);
+      fprintf(stderr, "rown=%"dFIDX", out.len=%"dFIDX"\n",
+	       rown,  (*out).len);
       return FAIL;
     }
 
@@ -1183,9 +1212,9 @@ int sparse_row_tim_vec(struct sparse *K, FIDX rown, FIDX *rows,
       if ((rows[i]<0)||(rows[i]>=row_max))
 	{
 	  fprintf(stderr, "sparse_row_tim_vec: "
-		  "rows[%d]=%d out of range!\n",
-		  (int) i, (int) rows[i]);
-	  fprintf(stderr, "row_max=%d\n", (int) row_max);
+		  "rows[%"dFIDX"]=%"dFIDX" out of range!\n",
+		   i,  rows[i]);
+	  fprintf(stderr, "row_max=%"dFIDX"\n",  row_max);
 	  return FAIL;
 	}
 
@@ -1276,8 +1305,8 @@ int sparse_row_tim_vec_add(struct sparse *K, FIDX rown, FIDX *rows,
     {
       fprintf(stderr, "sparse_row_tim_vec_add: "
 	      "supplied vector out has wrong size\n");
-      fprintf(stderr, "rown=%d, out.len=%d\n",
-	      (int) rown, (int) (*out).len);
+      fprintf(stderr, "rown=%"dFIDX", out.len=%"dFIDX"\n",
+	       rown,  (*out).len);
       return FAIL;
     }
 
@@ -1287,9 +1316,9 @@ int sparse_row_tim_vec_add(struct sparse *K, FIDX rown, FIDX *rows,
       if ((rows[i]<0)||(rows[i]>=row_max))
 	{
 	  fprintf(stderr, "sparse_row_tim_vec_add: "
-		  "rows[%d]=%d out of range!\n",
-		  (int) i, (int) rows[i]);
-	  fprintf(stderr, "row_max=%d\n", (int) row_max);
+		  "rows[%"dFIDX"]=%"dFIDX" out of range!\n",
+		   i,  rows[i]);
+	  fprintf(stderr, "row_max=%"dFIDX"\n",  row_max);
 	  return FAIL;
 	}
 
@@ -1390,9 +1419,9 @@ int sparse_mul_mat_vec_add_trans(struct sparse *K, struct vector *vec,
 	  if (col>=col_max)
 	    {
 	      fprintf(stderr, "sparse_mul_mat_vec_add_trans: "
-		      "invalid col>=col_max, row=%d  nnz=%d "
-		      "col=%d col_max=%d\n",
-		      (int) i, (int) j, (int) col, (int) col_max);
+		      "invalid col>=col_max, row=%"dFIDX"  nnz=%"dFIDX" "
+		      "col=%"dFIDX" col_max=%"dFIDX"\n",
+		       i,  j,  col,  col_max);
 	      return FAIL;
 	    }
 #endif
@@ -1447,7 +1476,7 @@ int sparse_add_local(struct sparse *K, enum transposetype trans,
 	      {
 		fprintf(stderr, "sparse_add_local: "
 			"coulnd get/create entry in K for "
-			"row[%d] col[%d] ", (int) i, (int) j);
+			"row[%"dFIDX"] col[%"dFIDX"] ",  i,  j);
 		return FAIL;
 	      }
 	    (*entry)+=local[j*LDlocal+i];
@@ -1463,7 +1492,7 @@ int sparse_add_local(struct sparse *K, enum transposetype trans,
 	      {
 		fprintf(stderr, "sparse_add_local: "
 			"coulnd get/create entry in K for "
-			"row[%d] col[%d] ", (int) i, (int) j);
+			"row[%"dFIDX"] col[%"dFIDX"] ",  i,  j);
 		return FAIL;
 	      }
 	    (*entry)+=local[i*LDlocal+j];
@@ -1509,8 +1538,8 @@ int sparse_get_entry(struct sparse *K, FIDX row, FIDX col,
   if ((row>=(*K).row_nr)||(row<0))
     {
       fprintf( stderr, "sparse_get_entry: "
-	       "row=%d >= K.row_nr=%d  !\n",
-	       (int) row, (int) (*K).row_nr);
+	       "row=%"dFIDX" >= K.row_nr=%"dFIDX"  !\n",
+	        row,  (*K).row_nr);
       return FAIL;
     }
 
@@ -1594,9 +1623,9 @@ int sparse_get_entry(struct sparse *K, FIDX row, FIDX col,
 	    if (create==1)
 	      {
 		fprintf( stderr, "sparse_get_entry: "
-			 "K(%d,%d) does not exist \n"
+			 "K(%"dFIDX",%"dFIDX") does not exist \n"
 			 "can not be created in compressed row mode!\n",
-			 (int) row, (int) col);
+			  row,  col);
 	      }
 	    (*entry)=NULL;
 	    return FAIL;
@@ -1631,7 +1660,6 @@ int sparse_convert_compressed_row(struct sparse *K
 */
 				  ){
   FIDX i, j, endrow, row_nr;
-  int err;
 
   FIDX *Arows, *Acols;
   double *A;
@@ -1767,7 +1795,7 @@ int sparse_extract_invdiag(struct sparse *K, struct vector *Kinvdiag
       else
 	{
 	  fprintf(stderr, "sparse_extract_invdiag: K has zero diagonal "
-		  "entry! entry %d\n", (int) i);
+		  "entry! entry %"dFIDX"\n",  i);
 	  return FAIL;
 	}
     }
@@ -1872,8 +1900,8 @@ int vector_diag_scale(void *notused, struct vector *in,
   if ((in->len != out->len)||(in->len != diag->len))
     {
       fprintf(stderr, "vector_diag_scale: vector sizes don't match!\n");
-      fprintf(stderr, "diag.len=%d, vec.len=%d!\n",
-	      (int) diag->len, (int) in->len);
+      fprintf(stderr, "diag.len=%"dFIDX", vec.len=%"dFIDX"!\n",
+	       diag->len,  in->len);
       return FAIL;
     }
 
@@ -2280,7 +2308,8 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
 		    numbers */
   FIDX dofs;     /* number of DOFs  */
 
-  int bandw;    /* the bandwidth of the matrix */
+  FIDX bandw;    /* the bandwidth of the matrix */
+  int  i_bandw;  /* integer copy of bandw */
 
   /* some vars for Fortran calls */
   char uplo;
@@ -2301,7 +2330,7 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
 
 
   dofs= (*K).row_nr;
-  /* printf("dof %d\n",dofs); */
+  /* printf("dof %"dFIDX"\n",dofs); */
   /* allocate memory for the local data */
   TRY_MALLOC( xadj, (dofs+1), FIDX, coarse_mat_set);
   TRY_MALLOC( adjncy, Arows[dofs], FIDX, coarse_mat_set);
@@ -2375,7 +2404,7 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
   if (xadj[0]!=bandw)
     {
       fprintf(stderr, "coarse_mat_set: bandw_reduction failed\n"
-	      "real bandwidth=%d     claimed=%d\n", (int) xadj[0], bandw);
+	      "real bandwidth=%"dFIDX"     claimed=%"dFIDX"\n",  xadj[0], bandw);
       return FAIL;
     }
 #endif
@@ -2439,10 +2468,10 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
 #ifdef DEBUGFEINS
 	      if (abs(row-col)>bandw)
 		{
-		  fprintf(stderr, "coarse_mat_set: bandw=%d "
-			  "seems to be wrong: row=%d col=%d  "
+		  fprintf(stderr, "coarse_mat_set: bandw=%"dFIDX" "
+			  "seems to be wrong: row=%"dFIDX" col=%"dFIDX"  "
 			  "symm=%d\n", bandw,
-			  (int) row, (int) col, symm);
+			   row,  col, symm);
 		  return FAIL;
 		}
 #endif
@@ -2485,9 +2514,17 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
 	  (*cmat).A[col*LDA+bandw+row-col]=1.0;
 	}
       uplo='U';
-      fdofs = (int) dofs;
-      fLDA  = (int) LDA;
-      dpbtrf_( &uplo, &fdofs, &bandw, (*cmat).A, &fLDA, &err); 
+      fdofs =  dofs;
+      fLDA  =  LDA;
+      i_bandw= (int) bandw;
+      if ((FIDX)i_bandw!= bandw)
+	{
+	  fprintf(stderr, 
+		  "coarse_mat_set: integer overflow in bandw\n");
+	  return FAIL;
+	}
+
+      dpbtrf_( &uplo, &fdofs, &i_bandw, (*cmat).A, &fLDA, &err); 
       if (err!=0)
 	{
 	  fprintf(stderr, 
@@ -2513,10 +2550,10 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
 #ifdef DEBUGFEINS
 	      if (abs(row-col)>bandw)
 		{
-		  fprintf(stderr, "coarse_mat_set: bandw=%d "
-			  "seems to be wrong: row=%d col=%d  "
+		  fprintf(stderr, "coarse_mat_set: bandw=%"dFIDX" "
+			  "seems to be wrong: row=%"dFIDX" col=%"dFIDX"  "
 			  "symm=%d\n", bandw, 
-			  (int) row, (int) col, symm);
+			   row,  col, symm);
 		  return FAIL;
 		}
 #endif
@@ -2548,9 +2585,10 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
 	  col=row;
 	  (*cmat).A[col*LDA+2*bandw+row-col]=1.0;
 	}
-      fdofs = (int) dofs;
-      fLDA  = (int) LDA;
-      dgbtrf_( &fdofs, &fdofs, &bandw, &bandw, (*cmat).A, &fLDA,
+      fdofs =  dofs;
+      fLDA  =  LDA;
+      i_bandw= (int) bandw;
+      dgbtrf_( &fdofs, &fdofs, &i_bandw, &i_bandw, (*cmat).A, &fLDA,
 	       (*cmat).ipiv, &err); 
       if (err!=0)
 	{
@@ -2603,8 +2641,8 @@ int coarse_mat_solve( struct coarse_mat *cmat, enum transposetype trans,
            FAIL    - failure, see error message
 */
 		      ){
-  FIDX i, dof;
-  int LDA, bandw, symm, nr, one, info;
+  FIDX i, dof, bandw;
+  int LDA, i_bandw, symm, nr, one, info;
   char uplo='U';
 
   nr    = (*cmat).nr;
@@ -2629,9 +2667,10 @@ int coarse_mat_solve( struct coarse_mat *cmat, enum transposetype trans,
   switch(symm)
     {
     case 1:
-      LDA=bandw+1;
+      LDA=(int) bandw+1;
+      i_bandw=(int) bandw;
       /* call LAPACK to do the solve */
-      dpbtrs_(&uplo, &nr, &bandw, &one, (*cmat).A, &LDA, (*cmat).help,
+      dpbtrs_(&uplo, &nr, &i_bandw, &one, (*cmat).A, &LDA, (*cmat).help,
 	      &nr, &info);
       if (info!=0)
 	{
@@ -2642,13 +2681,14 @@ int coarse_mat_solve( struct coarse_mat *cmat, enum transposetype trans,
 	}
       break;
     case 2:
-      LDA=3*bandw+1;
+      LDA=(int) 3*bandw+1;
+      i_bandw=(int) bandw;
       
       if (trans==NoTrans) uplo='N';
       else uplo='T';
 
       /* call LAPACK to do the solve */
-      dgbtrs_(&uplo, &nr, &bandw, &bandw, &one, (*cmat).A, &LDA,
+      dgbtrs_(&uplo, &nr, &i_bandw, &i_bandw, &one, (*cmat).A, &LDA,
 	      (*cmat).ipiv, (*cmat).help, &nr, &info);
       if (info!=0)
 	{
@@ -2716,7 +2756,7 @@ void int_list_free( struct int_list *L
 /*FUNCTION*/
 int bandw_red_perm( FIDX nr, FIDX *adix, FIDX *adel,
 		    FIDX *perm, FIDX *iperm,
-		    int *bandw
+		    FIDX *bandw
 /* calls the Gibbs-Poole-Stockmeyer modification of the
    Cuthill-McKee algorithm to produce a bandwith reducing permutaion
    for a given adjency list of a sparce matrix
@@ -2859,7 +2899,7 @@ int bandw_red_perm( FIDX nr, FIDX *adix, FIDX *adel,
       FUNCTION_FAILURE_HANDLE(err, bandw_gipost, bandw_red_perm);
 
       /* overall bandwidth is max of subgraph bandwidths */
-      if (sbandw>*bandw) *bandw=(int) sbandw;
+      if (sbandw>*bandw) *bandw= sbandw;
     }
 
   /* the element list parts.el itself forms a permutation, which
@@ -2962,7 +3002,7 @@ int bandw_gipost( FIDX n1, FIDX n2, struct int_list *ad,
 	v=i;
       }
 
-/*   printf(" test: min degree=%d\n", tmp); */
+/*   printf(" test: min degree=%"dFIDX"\n", tmp); */
 
   err=bandw_level_struct( v, n1, n2, ad, Lv, hint);
   FUNCTION_FAILURE_HANDLE(err, bandw_level_struct, bandw_gipost);
@@ -3049,7 +3089,7 @@ int bandw_gipost( FIDX n1, FIDX n2, struct int_list *ad,
 	    }
 	}
     }
-  /* printf("  test: minwidth=%d  for  Li[%d]\n", widthu, u); /* */
+  /* printf("  test: minwidth=%"dFIDX"  for  Li[%"dFIDX"]\n", widthu, u); /* */
   /* by now u is the number of the Li width the smallest width, so
      define Lu and u correctly */
   Lu=Li[u];
@@ -3117,7 +3157,7 @@ int bandw_gipost( FIDX n1, FIDX n2, struct int_list *ad,
     {
       if (hintv[i]==length-hintu[i])
 	{
-	  /* printf("  test:   vertex[%3d]: lvl=%d\n", i, hintv[i]);
+	  /* printf("  test:   vertex[%3d]: lvl=%"dFIDX"\n", i, hintv[i]);
 	  /* take it out of the game and count it in this level */
 	  hint[i]=-1;
 	  Nix[hintv[i]]++;
@@ -3143,7 +3183,7 @@ int bandw_gipost( FIDX n1, FIDX n2, struct int_list *ad,
 				  bandw_gipost);
 	}
     }
-/*   printf("  test: %d connected subgraphs need to be assigned to a level\n", */
+/*   printf("  test: %"dFIDX" connected subgraphs need to be assigned to a level\n", */
 /* 	 nc); */
   
   TRY_MALLOC(sorter , 2*nc, FIDX, bandw_gipost);
@@ -3205,8 +3245,8 @@ int bandw_gipost( FIDX n1, FIDX n2, struct int_list *ad,
 	  if (widthv<=widthu) choose=1;
 	  else choose=2;
 	}
-/*       printf(" test:  choose=%d  Hixmax=%d, Lixmax=%d    " */
-/* 	     "widthv=%d, widthu=%d\n", */
+/*       printf(" test:  choose=%"dFIDX"  Hixmax=%"dFIDX", Lixmax=%"dFIDX"    " */
+/* 	     "widthv=%"dFIDX", widthu=%"dFIDX"\n", */
 /* 	     choose, Hixmax, Lixmax, widthv, widthu); */
       /* now remember what we have decided by modifying hintv and
 	 hintu */
@@ -3302,8 +3342,8 @@ int bandw_gipost( FIDX n1, FIDX n2, struct int_list *ad,
 
   /* for all levels: assign the adjecent vertices of the lower level
      first with numbers, then the remaining of the higher level */
-/*   printf(" test:  start numbering from=%d  to=%d  dir=%+d   " */
-/* 	 "n1=%d  current=%d\n", from, to, dir, n1, current); */
+/*   printf(" test:  start numbering from=%"dFIDX"  to=%"dFIDX"  dir=%+d   " */
+/* 	 "n1=%"dFIDX"  current=%"dFIDX"\n", from, to, dir, n1, current); */
   for (lvl=from+dir; dir*lvl<=dir*to; lvl+=dir)
     {
       /* lower -> higher */
@@ -3314,23 +3354,23 @@ int bandw_gipost( FIDX n1, FIDX n2, struct int_list *ad,
       err=bandw_number_adj_vert( ad, Lv, hintv, lvl, lvl, n1, n2,
 				 perm, iperm, &current, sorter);
       FUNCTION_FAILURE_HANDLE( err, bandw_number_adj_vert, bandw_gipost);
-/*       printf(" test:     lvl=%d   current=%d\n", */
+/*       printf(" test:     lvl=%"dFIDX"   current=%"dFIDX"\n", */
 /* 	     lvl, current); */
     }
 
 
 
-/*   printf(" test: gipost done,  n2=%d   ?=  current=%d\n", n2, current); */
+/*   printf(" test: gipost done,  n2=%"dFIDX"   ?=  current=%"dFIDX"\n", n2, current); */
 
 #ifdef DEBUGFEINS
   if (current!=n2)
     {
       fprintf(stderr,"bandw_gipost: failed to assign all vertices \n");
       for (i=n1; i<n2; i++)
-	printf(" vertex %3d  assigned  %3d, level=%3d\n",
-		  (int) i, (int) perm[i], (int) hintv[i]);
+	printf(" vertex %3"dFIDX"  assigned  %3"dFIDX", level=%3"dFIDX"\n",
+		   i,  perm[i],  hintv[i]);
       /*if (perm[i]==-1)
-	fprintf(stderr," vertex %d  not assigned, level=%d\n",
+	fprintf(stderr," vertex %"dFIDX"  not assigned, level=%"dFIDX"\n",
 	i, hintv[i]);*/
       return FAIL;
     }
@@ -3458,7 +3498,7 @@ int bandw_level_struct( FIDX root, FIDX n1, FIDX n2, struct int_list *ad,
   /* if the last level is empty, reduce the number of levels again */
   if (L->ix[L->ixn]==L->ix[L->ixn+1]) L->ixn--;
 
-  /* printf("  test: levels=%3d, el in last level=%3d\n", L->ixn,
+  /* printf("  test: levels=%3"dFIDX", el in last level=%3"dFIDX"\n", L->ixn,
      L->ix[L->ixn+1]-L->ix[L->ixn]); /* */
 
 #ifdef DEBUGFEINS
@@ -3466,8 +3506,8 @@ int bandw_level_struct( FIDX root, FIDX n1, FIDX n2, struct int_list *ad,
     if (hint[i]==-1)
       {
 	fprintf(stderr,"bandw_level_struct: node with no level found\n"
-		"node=%d, n1=%d, n2=%d, root=%d \n",
-		(int) (i+n1), (int) n1, (int) n2, (int) root);
+		"node=%"dFIDX", n1=%"dFIDX", n2=%"dFIDX", root=%"dFIDX" \n",
+		 (i+n1),  n1,  n2,  root);
 	
 	return FAIL;
       }
@@ -3506,7 +3546,6 @@ int bandw_mark_connected( FIDX label, FIDX start, FIDX end, FIDX *adix,
                      invalid,
 */
 			){
-  int  err;
   FIDX i, j;
 
   FIDX *rec_start, *rec_j; 
@@ -3536,8 +3575,8 @@ int bandw_mark_connected( FIDX label, FIDX start, FIDX end, FIDX *adix,
 	  if ((marked[i-n1]>0)&&(marked[i-n1]!=label))
 	    {
 	      fprintf(stderr, "bandw_mark_connected: found vertex allready "
-		      "marked[%d]=%d  != %d=label\n",
-		      (int) i, (int) marked[i-n1], (int) label);
+		      "marked[%"dFIDX"]=%"dFIDX"  != %"dFIDX"=label\n",
+		       i,  marked[i-n1],  label);
 	      return FAIL;
 	    }
 	  if (marked[i-n1]==0)
@@ -3635,7 +3674,7 @@ int bandw_number_adj_vert(struct int_list *ad, struct int_list *Lv,
     }
 #endif
 
-  /* printf(" test: lowest_l1= %3d   l1=%3d   l2=%3d\n", lowest_l1,
+  /* printf(" test: lowest_l1= %3"dFIDX"   l1=%3"dFIDX"   l2=%3"dFIDX"\n", lowest_l1,
      l1, l2); /* */
 
   /* as long as there apear unnumbered nodes in the level... */
@@ -3648,7 +3687,7 @@ int bandw_number_adj_vert(struct int_list *ad, struct int_list *Lv,
       if ((iperm[lowest_l1]>=0)&&(level[iperm[lowest_l1]-n1]==l1))
 	{
 	  i=iperm[lowest_l1];
-	  /* printf(" test: check connected to %d:", i); /* */
+	  /* printf(" test: check connected to %"dFIDX":", i); /* */
 	  /* set sorter length (sl) to 0 = number of found unnumbered
 	     adjecent vertices */
 	  sl=0;
@@ -3657,7 +3696,7 @@ int bandw_number_adj_vert(struct int_list *ad, struct int_list *Lv,
 	    {
 	      FIDX adj_vx;
 	      adj_vx=ad->el[j];
-	  /* printf("   test: adj_vx=%3d    level[adj]=%3d   perm[adj]=%3d\n",
+	  /* printf("   test: adj_vx=%3"dFIDX"    level[adj]=%3"dFIDX"   perm[adj]=%3"dFIDX"\n",
 		 adj_vx, level[adj_vx-n1], perm[adj_vx]); /* */
 	      /* if in level l2 and not numbered */
 	      if ((level[adj_vx-n1]==l2)&&(perm[adj_vx]==-1))
@@ -3671,7 +3710,7 @@ int bandw_number_adj_vert(struct int_list *ad, struct int_list *Lv,
 		  sl++;
 		}
 	    }
-	  /* printf("  %3d are connected\n",sl); /* */
+	  /* printf("  %3"dFIDX" are connected\n",sl); /* */
 	  /* if vertices are found, sort them by degree, number them
 	     in increasing order */
 	  if (sl>0)
@@ -3681,7 +3720,7 @@ int bandw_number_adj_vert(struct int_list *ad, struct int_list *Lv,
 		{
 		  FIDX tv;
 		  tv=sorter[j*2+1];
-		  /* printf(" test: sorter[%3d]= [ %3d, %3d ]   current=%3d\n",
+		  /* printf(" test: sorter[%3"dFIDX"]= [ %3"dFIDX", %3"dFIDX" ]   current=%3"dFIDX"\n",
 		     j, sorter[j*2], sorter[j*2+1], *current); /* */
 		  perm[tv]=*current;
 		  iperm[*current]=tv;
@@ -3692,8 +3731,8 @@ int bandw_number_adj_vert(struct int_list *ad, struct int_list *Lv,
 		    {
 		      fprintf(stderr, "bandw_number_adj_vert: "
 			      "something wrong, try to number over "
-			      "limit: current=%d   n2=%d\n",
-			      (int) *current, (int) n2);
+			      "limit: current=%"dFIDX"   n2=%"dFIDX"\n",
+			       *current,  n2);
 		      return FAIL;
 		    }
 #else 
@@ -3757,8 +3796,8 @@ int bandw_number_adj_vert(struct int_list *ad, struct int_list *Lv,
 		{
 		  fprintf(stderr, "bandw_number_adj_vert: "
 			  "something wrong, try to number over "
-			  "limit: current=%d   n2=%d\n",
-			  (int) *current, (int) n2);
+			  "limit: current=%"dFIDX"   n2=%"dFIDX"\n",
+			   *current,  n2);
 		  return FAIL;
 		}
 #else 
