@@ -56,7 +56,7 @@ int sparse_flex_alloc(struct sparse *K, FIDX nrows
            FAIL    - failure, see error message
 */);
 
-int sparse_alloc(struct sparse *K, FIDX nrows, FIDX density
+int sparse_alloc(struct sparse *K, FIDX nrows, FIDX notused_density
 /* OBSOLETE, kept for ease of switching to new matrix format;
    creates a sparse matrix in the initial flexible format with nrows rows 
    
@@ -630,7 +630,7 @@ void int_list_free( struct int_list *L
 
 int bandw_red_perm( FIDX nr, FIDX *adix, FIDX *adel,
 		    FIDX *perm, FIDX *iperm,
-		    int *bandw
+		    FIDX *bandw
 /* calls the Gibbs-Poole-Stockmeyer modification of the
    Cuthill-McKee algorithm to produce a bandwith reducing permutaion
    for a given adjency list of a sparce matrix
