@@ -47,10 +47,6 @@ TO_HEADER:
 */
 
 
-
-/* function prototypes */
-#include "stokesassem.h"
-
 /* prototypes of external functions */
 #include <math.h>
 #include "elements.h"
@@ -61,9 +57,10 @@ TO_HEADER:
 #include "navsto_aux.h"
 #include "feins_lapack.h"    /* link against LAPACK! */
 
-#define FBLOCKDENSITY 32
-#define BBLOCKDENSITY 32
-#define MBLOCKDENSITY 12
+
+/* function prototypes */
+#include "stokesassem.h"
+
 
 /*FUNCTION*/
 int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
@@ -217,8 +214,8 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
   if (noFs!=1)
     {
       Fs=&(*K).Fs[lvl];
-      err=sparse_alloc( Fs, vx_nr, FBLOCKDENSITY);
-      FUNCTION_FAILURE_HANDLE( err, sparse_alloc, stokes_assem_t21);
+      err=sparse_flex_alloc( Fs, vx_nr);
+      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
 
       TRY_MALLOC( (*K).mg, 1, struct mgdata, stokes_assem_t21);
     }
@@ -226,11 +223,11 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
 
   for (i=0; i<dim; i++)
     {
-      err=sparse_alloc( &(*K).Bs[i], pvx_nr, BBLOCKDENSITY);
-      FUNCTION_FAILURE_HANDLE( err, sparse_alloc, stokes_assem_t21);
+      err=sparse_flex_alloc( &(*K).Bs[i], pvx_nr);
+      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
     }
-  err=sparse_alloc( &(*K).M, pvx_nr, MBLOCKDENSITY);
-  FUNCTION_FAILURE_HANDLE( err, sparse_alloc, stokes_assem_t21);
+  err=sparse_flex_alloc( &(*K).M, pvx_nr);
+  FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
 
 
   /* initialise rhs, u0 */
@@ -357,8 +354,9 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
 	  dgemm_( &fNoTrans, &fTrans, &fbas_n1, &fbas_n1, &fone,
 		  &dhelp, &phi1[k*bas_n1], &fbas_n1, &phi1[k*bas_n1],
 		  &fbas_n1, &done, elM, &fbas_n1 );
-
-#warning stokes_t21: no rhs assembly yet!
+#ifdef FEINS_have_warning 
+#warning "stokes_t21: no rhs assembly yet!"
+#endif
 	} /* end loop over all integration points */
       
       /* elF, elB and elM are ready, add to K
@@ -398,8 +396,9 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
 
     } /* end loop over all elements */
 
-
-#warning stokes_T21: Neumann BC not accounted for in rhs yet!
+#ifdef FEINS_have_warning 
+#warning "stokes_T21: Neumann BC not accounted for in rhs yet!"
+#endif
   /* first Neumann BC, otherwise they would collide with Dirichlet BC
    */
 
@@ -551,13 +550,13 @@ int stokes_diri_bc_init_t2( struct mesh *m, struct vector *uinit,
 	  fu=(*m).bound[bd*bd_w + MCT2BDFNCT];
 	  err1=mesh_func_eval(m, fu,
 			      &(*m).vertex[node1*vx_w+MCT2VXSTRT],
-			      2, &lu0[0] );
+			      0.0, 2, &lu0[0], NULL, NULL );
 	  err2=mesh_func_eval(m, fu,
 			      &(*m).vertex[node2*vx_w+MCT2VXSTRT],
-			      2, &lu0[2]);
+			      0.0, 2, &lu0[2], NULL, NULL);
 	  errm=mesh_func_eval(m, fu,
 			      &(*m).vertex[nodem*vx_w+MCT2VXSTRT],
-			      2, &lu0[4]);
+			      0.0, 2, &lu0[4], NULL, NULL);
 	  if (((err1!=SUCCESS)||(err2)!=SUCCESS)||((errm)!=SUCCESS))
 	    {
 	      fprintf(stderr, "stokes_diri_bc_init_t2: "
