@@ -52,7 +52,7 @@
 #define __USE_GNU
 #endif
 
-#include <unistd.h>
+//#include <unistd.h>
 #include <string.h>
 
 int main(int argc, char *argv[])
@@ -167,7 +167,7 @@ int main(int argc, char *argv[])
   err=vector_alloc( &nodem, dim*vx_nr );
   FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);  
   for (i=0; i<dim*vx_nr; i++) nodem.V[i]=0.0;    
-  printf("\nLevel %d, vx_nr = %d\n", 0, msh2.vx_nr);	
+  printf("\nLevel %d, vx_nr = %"dFIDX"\n", 0, msh2.vx_nr);	
 
   /* Mesh Refinement */
   for (lvl=0; lvl<lvlm; lvl++)
@@ -175,7 +175,7 @@ int main(int argc, char *argv[])
       if(msh2.lvl+1!=lvl)
         {
 	  printf("Level of Mesh doesn't match with refinement level!"
-		 " (%d+1!= %d)\n", msh2.lvl, lvl);
+		 " (%"dFIDX"+1!= %"dFIDX")\n", msh2.lvl, lvl);
 	  return FAIL;
         }
       err=sparse_flex_alloc( &Ks[lvl], dim*vx_nr);
@@ -213,16 +213,17 @@ int main(int argc, char *argv[])
 	  coarse_mat_free(&K_inv);
 	}
       if ((set.solver==1)||(set.solver==2))
-        { /* initialize multigrid data for PCG_MG or PCG_BPX */
+        {
+	  struct coarse_mat *cmat;
+	  FIDX *coarse_bc_nodes;
+	  FIDX n_c_bc=0;
+      /* initialize multigrid data for PCG_MG or PCG_BPX */
 	  err=multilvl_init_tx( &msh2, dim, &ml, 2);
 	  FUNCTION_FAILURE_HANDLE( err, multilvl_init_tx, main);
 	  err=mg_init_tx( Ks, &msh2, &ml, &mg, &P);
 	  FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
 	
 	  /* generate the coarse grid matrix */
-	  struct coarse_mat *cmat;
-	  FIDX *coarse_bc_nodes;
-	  FIDX n_c_bc=0;
 	  TRY_MALLOC(cmat, 1,  struct coarse_mat, main);
 	  TRY_MALLOC(coarse_bc_nodes, P.len, FIDX, main);
 	  for (i=0; i<P.len; i++)
@@ -324,7 +325,7 @@ int main(int argc, char *argv[])
       if(0==1) /* PRINT DEFORMATION IN GIVEN POINT */
 	{
 	  j = msh2.pccrit[MCXXPCDAT1];
-	  printf("nodem (%d): %8.6e, %8.6e\n", 
+	  printf("nodem (%"dFIDX"): %8.6e, %8.6e\n", 
 		 j, nodem.V[0*vx_nr+j], nodem.V[1*vx_nr+j] );
 	}
     
@@ -333,9 +334,13 @@ int main(int argc, char *argv[])
         {
 	  /* error estimator */
 	  FIDX *marker;
+	  FIDX nrmarked;
+	  double globest;
+
 	  TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
-          err=error_est_lame_residual_t2(&msh2, &nodem, lambda, mu, 
-					 marker, &set, 2);  
+          err=lame_error_est_residual_t2(&msh2, &nodem, lambda, mu, 
+					 marker, &nrmarked, &globest,
+					 &set, 2);  
           FUNCTION_FAILURE_HANDLE( err, error_est_lame_residual_t2, main);
 	  // for (i=0; i<msh2.el_nr; i++) marker[i] = 1;
 	  err=mesh_refine_adaptive_t2( &msh2, marker);
@@ -352,13 +357,14 @@ int main(int argc, char *argv[])
       if(msh2.lvl!=lvl)
         {
 	  printf("Level of Mesh doesn't match with refinement level!"
-		 " (%d != %d)\n", msh2.lvl, lvl);
+		 " (%"dFIDX" != %"dFIDX")\n", msh2.lvl, lvl);
 	  return FAIL;
         }
-      printf("\nLevel %d, vx_nr = %d\n", lvl+1, msh2.vx_nr);
+      printf("\nLevel %"dFIDX", vx_nr = %"dFIDX"\n", lvl+1, msh2.vx_nr);
 
       /* interpolate the solution vector nodem to the finer grid */
         {	
+	  FIDX node;
 	  /* redefine multilvl data */
 	  err=multilvl_init_tx( &msh2, dim, &ml, 2);
 	  FUNCTION_FAILURE_HANDLE( err, multilvl_init_tx, main);
@@ -366,7 +372,6 @@ int main(int argc, char *argv[])
 	  FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
 
 
-	  FIDX node;
 	  /* Remark: vx_nr is still the number of vertices on the
 	     coarser mesh (lvl), msh2.vx_nr is the number on the 
 	     finer mesh (lvl+1) */
@@ -460,12 +465,12 @@ int main(int argc, char *argv[])
       char name[100];
 
       /* is already in copressed row format */
-      sprintf(name,"visual/Lame_Mat_C_%07d_nodes.txt", vx_nr);
+      sprintf(name,"visual/Lame_Mat_C_%07"dFIDX"_nodes.txt", vx_nr);
       err=sparse_mat_write_file(&Ks[lvlm], name);
       FUNCTION_FAILURE_HANDLE( err, sparse_mat_write_file, main);
 
       /* node positions */
-      sprintf(name,"visual/Lame_vertes_%07d_nodes.txt", vx_nr);
+      sprintf(name,"visual/Lame_vertes_%07"dFIDX"_nodes.txt", vx_nr);
       err=vector_write_file(&vx_old, name);
       FUNCTION_FAILURE_HANDLE( err, vector_write_file, main);
     }
@@ -479,10 +484,11 @@ int main(int argc, char *argv[])
     }
   if(0==1) /* PRINT RESTRICTED VECTOR */
     { 
-	  printf("Test for restriction\n");  
 	  FIDX node1, node2;
-	  int fathers[vx_nr]; /* vector to check if other nodes 
+	  FIDX *fathers; /* vector to check if other nodes 
 				 than the fathers were changed */
+	  TRY_MALLOC(fathers,vx_nr,FIDX,main);
+	  printf("Test for restriction\n");  
 	  /* Remark: vx_nr is the number on the finer mesh (lvl+1) */
 	  for (i=0; i<vx_nr*dim; i++) nodem.V[i] = 1.0;
 	  for (i=0; i<vx_nr; i++) fathers[i]=0;
@@ -502,7 +508,7 @@ int main(int argc, char *argv[])
 	      
 	  for (j=ml.levlhindx[lvlm]; j<ml.levlhindx[lvlm+1]; j++)
             {   
-	      printf("el_hier %d\n", j);	    
+	      printf("el_hier %"dFIDX"\n", j);	    
 	      for (i=0; i<6; i++) 
 		{
 		  d=msh2.elhier[j*msh2.eh_w+MCT2EHFAT1+i];
@@ -530,14 +536,15 @@ int main(int argc, char *argv[])
 		  MLVLFINDENTRY(node2, i+1*vx_nr, lvlm-1, ml);
 		  if ((node1>-1)&&(node2>-1))
 		    {
-		      printf("ERROR: node1=%d, node2=%d \n"
-			      "  for vertex i=%d in level lvlm-1=%d\n",
-			      node1, node2, i, lvlm-1);
+		      printf("ERROR: node1=%"dFIDX", node2=%"dFIDX" \n"
+			     "  for vertex i=%"dFIDX
+			     " in level lvlm-1=%"dFIDX"\n",
+			     node1, node2, i, lvlm-1);
 		    }
 		  else
 		    {
 		      if ( (abs(mg.xl[node1]-1)>1e-8)||(abs(mg.xl[node2])>1e-8) )
-			printf("ERROR: x%d \thas changed!\t%f, \t%f\n", 
+			printf("ERROR: x%"dFIDX" \thas changed!\t%f, \t%f\n", 
 			       i, mg.xl[node1], mg.xl[node2]);
 		    }	
 		}
@@ -550,7 +557,7 @@ int main(int argc, char *argv[])
 	      MLVLFINDENTRY(node2, i+1*vx_nr, lvlm-1, ml);
 	      if ((node1>-1)&&(node2>-1))
 		{
-		  printf("restr[%3d]= %10f  %10f\n", i,
+		  printf("restr[%3"dFIDX"]= %10f  %10f\n", i,
 			 mg.xl[node1], mg.xl[node2]);
 		}
 	    }
@@ -561,10 +568,12 @@ int main(int argc, char *argv[])
     
   if(0==1) /* PRINT INTERPOLATED VECTOR */
     { 
-	  printf("Test for interpolation\n");  
 	  FIDX node1, node2;
-	  int fathers[vx_nr]; /* vector to check if other nodes 
+	  FIDX *fathers; /* vector to check if other nodes 
 				 than the fathers were changed */
+	  TRY_MALLOC(fathers,vx_nr,FIDX,main);
+	  printf("Test for interpolation\n");  
+	  
 	  /* Remark: vx_nr is the number on the finer mesh (lvl+1) */
 	  for (i=0; i<vx_nr*dim; i++) nodem.V[i] = 0.0;
 	  for (i=0; i<vx_nr; i++) fathers[i]=0;
@@ -583,7 +592,7 @@ int main(int argc, char *argv[])
 	      
 	  for (j=ml.levlhindx[lvlm]; j<ml.levlhindx[lvlm+1]; j++)
             {   
-	      printf("el_hier %d\n", j);	    
+	      printf("el_hier %"dFIDX"\n", j);	    
 	      for (i=0; i<6; i++) 
 		{
 		  d=msh2.elhier[j*msh2.eh_w+MCT2EHFAT1+i];
@@ -628,7 +637,7 @@ int main(int argc, char *argv[])
 		  if ((node1>-1)&&(node2>-1))
 		    {
 		      if ( (abs(mg.xl[node1]-0.0)>1e-8)||(abs(mg.xl[node2]-0.0)>1e-8) )
-			printf("ERROR: x%d \thas changed!\t%f, \t%f\n", 
+			printf("ERROR: x%"dFIDX" \thas changed!\t%f, \t%f\n", 
 			       i, mg.xl[node1], mg.xl[node2]);
 		    }	
 		}
@@ -642,7 +651,7 @@ int main(int argc, char *argv[])
 	      if ((node1>-1)&&(node2>-1))
 		{
 		  if ( (abs(mg.xl[node1]-0.0)>1e-8)||(abs(mg.xl[node2]-0.0)>1e-8) )
-		  printf("restr[%3d]= %10f  %10f\n", i,
+		  printf("restr[%3"dFIDX"]= %10f  %10f\n", i,
 			 mg.xl[node1], mg.xl[node2]);
 		}
 	    }
@@ -654,18 +663,18 @@ int main(int argc, char *argv[])
   {
     for (i=ml.levlhindx[lvlm]; i<ml.levlhindx[lvlm+1]; i++)
       {   
-	printf("\nelhier %d:\nfathers  ",i);
-	for (j=0; j<3; j++) printf("%d (%f %f), ", 
+	printf("\nelhier %"dFIDX":\nfathers  ",i);
+	for (j=0; j<3; j++) printf("%"dFIDX" (%f %f), ", 
 msh2.elhier[i*msh2.eh_w+MCT2EHFAT1+j], 
 msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHFAT1+j]*msh2.vx_w+MCT2VXSTRT+0],
 msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHFAT1+j]*msh2.vx_w+MCT2VXSTRT+1]);
 	printf("\nfathers  ");
-	for (j=3; j<6; j++) printf("%d (%f %f), ", 
+	for (j=3; j<6; j++) printf("%"dFIDX" (%f %f), ", 
 msh2.elhier[i*msh2.eh_w+MCT2EHFAT1+j],
 msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHFAT1+j]*msh2.vx_w+MCT2VXSTRT+0],
 msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHFAT1+j]*msh2.vx_w+MCT2VXSTRT+1]);	
 	printf("\nchildren ");
-	for (j=0; j<3; j++) printf("%d (%f %f), ", 
+	for (j=0; j<3; j++) printf("%"dFIDX" (%f %f), ", 
 msh2.elhier[i*msh2.eh_w+MCT2EHCHL1+j],
 msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHCHL1+j]*msh2.vx_w+MCT2VXSTRT+0],
 msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHCHL1+j]*msh2.vx_w+MCT2VXSTRT+1]);
@@ -677,7 +686,7 @@ msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHCHL1+j]*msh2.vx_w+MCT2VXSTRT+1]);
   
   for (i=0; i<0*vx_nr; i++)
     {
-      printf("x%d: (%f,%f)\n", i,
+      printf("x%"dFIDX": (%f,%f)\n", i,
 	msh2.vertex[i*msh2.vx_w+MCT2VXSTRT+0],
 	msh2.vertex[i*msh2.vx_w+MCT2VXSTRT+1]);
     } 
