@@ -42,6 +42,7 @@
 /*#include "meshdetails.h"*/
 #include "sparse.h"
 #include "lin_solver.h"
+#include "linsolve_umfpack.h"
 #include "assembly.h"
 #include "gen_aux.h"
 #include "mesh_deform.h"
@@ -51,7 +52,7 @@
 #define __USE_GNU
 #endif
 
-#include <unistd.h>
+/*#include <unistd.h>*/
 #include <string.h>
 
 
@@ -75,12 +76,14 @@ int main(int argc, char *argv[])
 
   struct solver_settings set;
 	  
-  int  err;
+  int  err, stop;
   FIDX dim, vx_nr, i, j, d;
 
   FIDX level0, lvlm, lvl;
 
   double mu, lambda;
+  double globest;
+  FIDX nrmarked;
   int stiffening;
   int vcycles=1;
   int smooths=4;
@@ -91,8 +94,11 @@ int main(int argc, char *argv[])
   double resi;
   double res_start;
 
+  FIDX *marker;
+
   char *buffer;
-  
+  FILE *out;
+
   res_start = -1.0;
   stiffening = 1;  /* ==1 for real Lame', 
                       ==0 for stiffening of small elements */
@@ -110,6 +116,14 @@ int main(int argc, char *argv[])
   err=mesh_read_file_t1( &msh1, &set, argv[1] ); /* */
   FUNCTION_FAILURE_HANDLE( err, mesh_read_file_t1, main);
     
+
+  if(set.write_ssegs!=0) /* write shape segments for visualisation */
+    {
+      err=mesh_write_ssegs_svg_tx( &msh1, 18, "visual/shape_segs", 1);
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_ssegs_svg_tx, main);  
+    }   
+
+
   /* set the goals and methods */
   level0  = set.refine_ini;
   lvlm   = set.refine_steps;
@@ -159,15 +173,16 @@ int main(int argc, char *argv[])
     {
       nodem.V[i]=0.0;    
     }
-  printf("\n Level = %d : vx_nr = %d\n", 0, msh2.vx_nr);	
+  printf("\n Level = %d : vx_nr = %"dFIDX"\n", 0,  msh2.vx_nr);	
       
   /* Mesh Refinement */
-  for (lvl=0; lvl<lvlm; lvl++)
+  stop=0;
+  for (lvl=0; (lvl<lvlm)&&(stop==0); lvl++)
     {  
       if(msh2.lvl+1!=lvl)
         {
 	  printf("Level of Mesh doesn't match with refinement level!"
-		 " (%d+1!= %d)\n", msh2.lvl, lvl);
+		 " (%"dFIDX"+1!= %"dFIDX")\n",  msh2.lvl,  lvl);
 	  return FAIL;
         }
       err=sparse_flex_alloc( &Ks[lvl], dim*vx_nr);
@@ -203,17 +218,25 @@ int main(int argc, char *argv[])
 
 	  coarse_mat_free(&K_inv);
 	}
+      if (set.solver==3) /* variant 3: direct solve with UMFPACK */
+	{
+	  err=sparse_solve_UMFPACK( &nodem, &Ks[lvl], &rhs, &P); /* */
+	  FUNCTION_FAILURE_HANDLE( err, sparse_solve_UMFPACK, main);
+	}
       if((set.solver==1)||(set.solver==2))
-	{/* set multilevel and multigrid data for PCG_MG or PCG_BPX */
+	{
+	  struct coarse_mat *cmat;
+	  FIDX *coarse_bc_nodes;
+	  FIDX n_c_bc;
+
+	  /* set multilevel and multigrid data for PCG_MG or PCG_BPX */
 	  err=multilvl_init_tx( &msh2, dim, &ml, 2);
 	  FUNCTION_FAILURE_HANDLE( err, multilvl_init_tx, main);
 	  err=mg_init_tx( Ks, &msh2, &ml, &mg, &P);
 	  FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
 	  
 	  /* generate the coarse grid matrix */
-	  struct coarse_mat *cmat;
-	  FIDX *coarse_bc_nodes;
-	  FIDX n_c_bc=0;
+	  n_c_bc=0;
 	  TRY_MALLOC(cmat, 1,  struct coarse_mat, main);
 	  TRY_MALLOC(coarse_bc_nodes, P.len, FIDX, main);
 	  for (i=0; i<P.len; i++)
@@ -292,66 +315,112 @@ int main(int argc, char *argv[])
 	  printf("PCG_BPX_refine |res|=%8.1e it=%3d\n", resi, iter);
 	  bpx_free(&bpx);
 	}
-      if((set.solver==1)||(set.solver==2))
-	{
-	  /* multigrid data and multilevel are no longer required */
-	  mg_free(&mg);
-	  multilvl_free(&ml);
-	}
-      /* rhs and projector are no longer required */
-      projector1_free(&P);
-      vector_free(&rhs);
 
       /* Apply mesh refinement */
-      if (set.refine_type==1)
-        { /* Adaptive mesh refinement */
-	  /* error estimator */
-	  FIDX *marker;
-	  TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
-	  err=error_est_lame_residual_t2(&msh2, &nodem, lambda, mu, marker,
-					 &set, 2); 
-	  FUNCTION_FAILURE_HANDLE( err, error_est_lame_residual_t2, main);
-	  /* refine the mesh */
-	  err=mesh_refine_adaptive_t2( &msh2, marker);
-          FUNCTION_FAILURE_HANDLE( err, mesh_refine_uniform_t2, main);
-	  free(marker);
-        }
-      else if (set.refine_type==0)
-        { /* Uniform mesh refinement */
-	  /* OPTIONAL: estimating the error */
-	  FIDX *marker;
-	  TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
-	  err=error_est_lame_residual_t2(&msh2, &nodem, lambda, mu,
-					 marker, &set, 2); 
-	  FUNCTION_FAILURE_HANDLE( err, error_est_lame_residual_t2, main);
-	  free(marker);
-	  /* refine the mesh */
-	  err=mesh_refine_uniform_t2( &msh2);
-          FUNCTION_FAILURE_HANDLE( err, mesh_refine_uniform_t2, main);
-        }
-      else
-	{
-	  printf("Error in main (test_lame.c): unknown refinement type!"
-		 " type = %d\n", set.refine_type);
-	  return FAIL;
-	}
-      if(msh2.lvl!=lvl)
+      {
+	FIDX *marker;
+	TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
+	switch(set.refine_type)
+	  {
+	  case 0:
+	    /* Uniform mesh refinement */
+	    /* OPTIONAL: estimating the error */
+	    err=lame_error_est_residual_t2(&msh2, &nodem, lambda, mu,
+					   marker, &nrmarked, &globest,
+					   &set, 2); 
+	    FUNCTION_FAILURE_HANDLE( err, lame_error_est_residual_t2, main);
+	    if ((globest<set.refine_stop)||(4*vx_nr>set.refine_max_vx))
+	      {
+		stop=1;
+		lvlm=lvl;
+	      }
+	    else
+	      {
+		/* refine the mesh */
+		err=mesh_refine_uniform_t2( &msh2);
+		FUNCTION_FAILURE_HANDLE( err, mesh_refine_uniform_t2, main);
+	      }
+	    break;
+	  case 1:
+	    /* Adaptive mesh refinement , residual error estimator */
+	    err=lame_error_est_residual_t2(&msh2, &nodem, lambda, mu,
+					   marker, &nrmarked, &globest,
+					   &set, 2); 
+	    FUNCTION_FAILURE_HANDLE( err, lame_error_est_residual_t2, main);
+
+	    if ((globest<set.refine_stop)||(vx_nr+4*nrmarked>set.refine_max_vx))
+	      {
+		stop=1;
+		lvlm=lvl;
+	      }
+	    else
+	      {
+		/* refine the mesh */
+		err=mesh_refine_adaptive_t2( &msh2, marker);
+		FUNCTION_FAILURE_HANDLE( err, mesh_refine_uniform_t2, main);
+	      }
+	    break;
+	  case 2:
+	    /* Adaptive mesh refinement, ZZ error estimator */
+	    err=gen_error_est_ZZ_tx(&msh2, dim, &nodem, 
+				    marker, &nrmarked, &globest, &set, 2); 
+	    FUNCTION_FAILURE_HANDLE( err, gen_error_est_ZZ_tx, main);
+
+	    if ((globest<set.refine_stop)||(vx_nr+4*nrmarked>set.refine_max_vx))
+	      {
+		stop=1;
+		lvlm=lvl;
+	      }
+	    else
+	      {
+		/* refine the mesh */
+		err=mesh_refine_adaptive_t2( &msh2, marker);
+		FUNCTION_FAILURE_HANDLE( err, mesh_refine_uniform_t2, main);
+	      }
+	    break;
+	  default:
+	    printf("Error in main (test_lame.c): unknown refinement type!"
+		   " type = %"dFIDX"\n",  set.refine_type);
+	    return FAIL;
+	  }
+	free(marker);
+      }
+      if ( ((stop==0)&&(msh2.lvl!=lvl))
+	   ||((stop==1)&&(msh2.lvl!=lvl-1)))
         {
 	  printf("Level of Mesh doesn't match with refinement level!"
-		 " (%d != %d)\n", msh2.lvl, lvl);
+		 " (%"dFIDX" != %"dFIDX")\n",  msh2.lvl,  lvl);
 	  return FAIL;
         }
-      printf("\n Level = %d : vx_nr = %d\n", lvl+1, msh2.vx_nr);
+      printf("\n Level = %"dFIDX" : vx_nr = %"dFIDX"    stop = %d\n",
+	      msh2.lvl,  msh2.vx_nr,  stop);
+
+
+      if (stop==0)
+        {
+	  /* free all solver data that is not valid for the refined
+	     mesh */
+	  if((set.solver==1)||(set.solver==2))
+	    {
+	      /* multigrid data and multilevel are no longer required */
+	      mg_free(&mg);
+	      multilvl_free(&ml);
+	    }
+	  /* rhs and projector are no longer required */
+	  projector1_free(&P);
+	  vector_free(&rhs);
+	}
 
       /* interpolate the solution vector nodem to the subsequent level */
+      if (stop==0)
         {
+	  FIDX node;
 	  /* redefine multilvl data */
 	  err=multilvl_init_tx( &msh2, dim, &ml, 2);
 	  FUNCTION_FAILURE_HANDLE( err, multilvl_init_tx, main);
 	  err=mg_init_tx( NULL, &msh2, &ml, &mg, NULL);
 	  FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
       
-	  FIDX node;
 	  /* Remark: vx_nr is still the number of vertices on the coarser
 	     mesh (lvl), msh2.vx_nr is the number on the finer mesh (lvl+1) */
 	  for (i=0; i<vx_nr; i++)
@@ -381,160 +450,157 @@ int main(int argc, char *argv[])
 	      }
 	  multilvl_free(&ml);	
 	  mg_free(&mg);
+
+	  /* adjust the new nodes to math the shape segments */
+	  err=mesh_sseg_adjust_tx( &msh2,  0,     0, NULL, NULL, 2);
+	  FUNCTION_FAILURE_HANDLE( err, mesh_sseg_adjust_tx, main); 
 	}
 
-	/* Simply adjust the new nodes to math the shape segments */
-	err=mesh_sseg_adjust_tx( &msh2,  0,     0, NULL, NULL, 2);
-	FUNCTION_FAILURE_HANDLE( err, mesh_sseg_adjust_tx, main); 
 
     } /* End of mesh refinement */
 
-
-  vx_nr=msh2.vx_nr; 
-  
   TRY_MALLOC(buffer, strlen(argv[1])+10, char, main);
 
-  /* allocate memory for matrix Ks, P and the vectors */
-  err=sparse_flex_alloc( &Ks[lvlm], dim*vx_nr);
-  FUNCTION_FAILURE_HANDLE( err, sparse_alloc, main);
-  err=projector1_alloc( &P, dim*vx_nr );
-  FUNCTION_FAILURE_HANDLE( err, projector1_alloc, main);
-  err=vector_alloc( &rhs, dim*vx_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
-  err=vector_alloc( &dIdu, dim*vx_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
-  err=vector_alloc( &psi, dim*vx_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
-  err=vector_alloc( &dIdx, dim*vx_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
-  err=vector_alloc( &dIdx_FD, dim*vx_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
-  err=vector_alloc( &dIdF, msh2.sp_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
-  
-  /* set vectors to zero */
-  for (i=0; i<dim*vx_nr; i++)
+  vx_nr=msh2.vx_nr; 
+
+  if (stop==0)
     {
-      rhs.V[i]  =0.0;
-      psi.V[i]  =0.0;
-      dIdu.V[i] =0.0;
-      dIdx.V[i]=0.0;
-      dIdx_FD.V[i]=0.0;
-    }
+      /* reached finest level because max levels exhausted,
+         need to solve once more */
+      /* allocate memory for matrix Ks, P and the vectors */
+      err=sparse_flex_alloc( &Ks[lvlm], dim*vx_nr);
+      FUNCTION_FAILURE_HANDLE( err, sparse_alloc, main);
+      err=projector1_alloc( &P, dim*vx_nr );
+      FUNCTION_FAILURE_HANDLE( err, projector1_alloc, main);
+      err=vector_alloc( &rhs, dim*vx_nr );
+      FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
 
-  /* assemble the system on the fine mesh */
-  err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &nodem,
-		     lambda, mu, stiffening, 2, 2);
-  FUNCTION_FAILURE_HANDLE( err, assem_lame_tx, main);
-
-
-  /* solve the equation system */
-  if (set.solver==0) /* variant 1: direct solve */
-    {
-      /* invert the stiffness matrix */
-      err=coarse_mat_set( &Ks[lvlm], P.len, P.V,
-			  1, &K_inv );
-      FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
-
-      /* project rhs */
-      for (i=0; i<P.len; i++) rhs.V[P.V[i]]=nodem.V[P.V[i]];
-
-      err=coarse_mat_solve( &K_inv, NoTrans, &rhs, &nodem);
-      FUNCTION_FAILURE_HANDLE(err, coarse_mat_solve, main);
-
-      coarse_mat_free(&K_inv);
-    }
-  if ((set.solver==1)||(set.solver==2))
-    { /* set multilevel and multigrid data for PCG_MG and PCG_BPX */
-      /* define the multigrid data */
-      err=multilvl_init_tx( &msh2, dim, &ml, 2);
-      FUNCTION_FAILURE_HANDLE( err, multilvl_init_tx, main);
-      err=mg_init_tx( Ks, &msh2, &ml, &mg, &P);
-      FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
-      
-      
-      mg.vcycles=vcycles;
-      mg.smooths=smooths;
-      if (stiffening==0)
-	mg.CGC_scale=0.25;
-      else
-	mg.CGC_scale=1.0;
-
-      /* generate the coarse grid matrix */
-      struct coarse_mat *cmat;
-      FIDX *coarse_bc_nodes;
-      FIDX n_c_bc=0;
-
-      TRY_MALLOC(cmat, 1,  struct coarse_mat, main);
-      TRY_MALLOC(coarse_bc_nodes, P.len, FIDX, main);
-      
-      for (i=0; i<P.len; i++)
+      /* set vectors to zero */
+      for (i=0; i<dim*vx_nr; i++)
 	{
-	  FIDX dof, child;
-	  dof=P.V[i];
-	  MLVLFINDENTRY(child, dof, 0, ml);
-	  if (child>=0)
+	  rhs.V[i]  =0.0;
+	}
+
+      /* assemble the system on the fine mesh */
+      err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &nodem,
+			    lambda, mu, stiffening, 2, 2);
+      FUNCTION_FAILURE_HANDLE( err, assem_lame_tx, main);
+
+
+      /* solve the equation system */
+      if (set.solver==0) /* variant 1: direct solve */
+	{
+	  /* invert the stiffness matrix */
+	  err=coarse_mat_set( &Ks[lvlm], P.len, P.V,
+			      1, &K_inv );
+	  FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
+
+	  /* project rhs */
+	  for (i=0; i<P.len; i++) rhs.V[P.V[i]]=nodem.V[P.V[i]];
+
+	  err=coarse_mat_solve( &K_inv, NoTrans, &rhs, &nodem);
+	  FUNCTION_FAILURE_HANDLE(err, coarse_mat_solve, main);
+
+	  coarse_mat_free(&K_inv);
+	}
+      if (set.solver==3) /* variant 3: direct solve with UMFPACK */
+	{
+	  err=sparse_solve_UMFPACK( &nodem, &Ks[lvlm], &rhs, &P); /* */
+	  FUNCTION_FAILURE_HANDLE( err, sparse_solve_UMFPACK, main);
+	}
+      if ((set.solver==1)||(set.solver==2))
+	{ 
+	  struct coarse_mat *cmat;
+	  FIDX *coarse_bc_nodes;
+	  FIDX n_c_bc;
+      /* set multilevel and multigrid data for PCG_MG and PCG_BPX */
+	  /* define the multigrid data */
+	  err=multilvl_init_tx( &msh2, dim, &ml, 2);
+	  FUNCTION_FAILURE_HANDLE( err, multilvl_init_tx, main);
+	  err=mg_init_tx( Ks, &msh2, &ml, &mg, &P);
+	  FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
+      
+      
+	  mg.vcycles=vcycles;
+	  mg.smooths=smooths;
+	  if (stiffening==0)
+	    mg.CGC_scale=0.25;
+	  else
+	    mg.CGC_scale=1.0;
+
+	  /* generate the coarse grid matrix */
+	  n_c_bc=0;
+	  TRY_MALLOC(cmat, 1,  struct coarse_mat, main);
+	  TRY_MALLOC(coarse_bc_nodes, P.len, FIDX, main);
+      
+	  for (i=0; i<P.len; i++)
 	    {
-	      coarse_bc_nodes[n_c_bc]=child-ml.nlevl[0+1];
-	      n_c_bc++;
+	      FIDX dof, child;
+	      dof=P.V[i];
+	      MLVLFINDENTRY(child, dof, 0, ml);
+	      if (child>=0)
+		{
+		  coarse_bc_nodes[n_c_bc]=child-ml.nlevl[0+1];
+		  n_c_bc++;
+		}
 	    }
-	}
-      err=coarse_mat_set( &Ks[0], n_c_bc, coarse_bc_nodes,
-			  1, cmat );
-      FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
-      mg.cmat=cmat;
+	  err=coarse_mat_set( &Ks[0], n_c_bc, coarse_bc_nodes,
+			      1, cmat );
+	  FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
+	  mg.cmat=cmat;
       
-      free(coarse_bc_nodes);
-    }
-  if (set.solver==1) /* variant 2: PCG_MG */
-    {
-      double rtol;
-      if (lvlm==0)
-	{
-	  rtol=set.solver_ini_rtol;
+	  free(coarse_bc_nodes);
 	}
-      else
+      if (set.solver==1) /* variant 2: PCG_MG */
 	{
-	  rtol=set.solver_ref_rtol;
+	  double rtol;
+	  if (lvlm==0)
+	    {
+	      rtol=set.solver_ini_rtol;
+	    }
+	  else
+	    {
+	      rtol=set.solver_ref_rtol;
+	    }
+	  err=PCG( 100, 3, set.solver_atol, rtol, 1, &nodem, &resi, &iter,
+		   sparse_mul_mat_vec, gen_proj_MG_tx,
+		   &Ks[lvlm], &rhs, &mg); /* */
+	  FUNCTION_FAILURE_HANDLE( err, PCG, main); /* */
+	  printf("PCG_MG_main   |res|=%8.1e it=%3d\n", resi, iter);
 	}
-      err=PCG( 100, 3, set.solver_atol, rtol, 1, &nodem, &resi, &iter,
-	       sparse_mul_mat_vec, gen_proj_MG_tx,
-	       &Ks[lvlm], &rhs, &mg); /* */
-      FUNCTION_FAILURE_HANDLE( err, PCG, main); /* */
-      printf("PCG_MG_main   |res|=%8.1e it=%3d\n", resi, iter);
-    }
-  if (set.solver==2) /* variant 3: PCG with BPX */
-    {
-      int iter;
-      double resi, rtol;
+      if (set.solver==2) /* variant 3: PCG with BPX */
+	{
+	  int iter;
+	  double resi, rtol;
       
-      err=bpx_init_tx(&bpx, &ml);
-      FUNCTION_FAILURE_HANDLE( err, bpx_init_tx, main);
-      bpx.msh = &msh2;
-      bpx.P   = &P;    
-      bpx.cmat = mg.cmat; /* */
-      if (lvlm==0)
-	{
-	  rtol=set.solver_ini_rtol;
+	  err=bpx_init_tx(&bpx, &ml);
+	  FUNCTION_FAILURE_HANDLE( err, bpx_init_tx, main);
+	  bpx.msh = &msh2;
+	  bpx.P   = &P;    
+	  bpx.cmat = mg.cmat; /* */
+	  if (lvlm==0)
+	    {
+	      rtol=set.solver_ini_rtol;
+	    }
+	  else
+	    {
+	      rtol=set.solver_ref_rtol;
+	    }
+	  err=PCG( 1000, 3, set.solver_atol, rtol, 1, &nodem, &resi, &iter,
+		   sparse_mul_mat_vec, gen_proj_bpx_tx,
+		   &Ks[lvl], &rhs, &bpx); /* */
+	  /* sparse_mat_write_file(&Ks[lvl], "visual/Lame_mat.txt"); /* */
+	  FUNCTION_FAILURE_HANDLE( err, PCG, main); /* */
+	  printf("PCG_BPX_refine |res|=%8.1e it=%3d\n", resi, iter);
+	  bpx_free(&bpx);
 	}
-      else
-	{
-	  rtol=set.solver_ref_rtol;
-	}
-      err=PCG( 1000, 3, set.solver_atol, rtol, 1, &nodem, &resi, &iter,
-	       sparse_mul_mat_vec, gen_proj_bpx_tx,
-	       &Ks[lvl], &rhs, &bpx); /* */
-      /* sparse_mat_write_file(&Ks[lvl], "visual/Lame_mat.txt"); /* */
-      FUNCTION_FAILURE_HANDLE( err, PCG, main); /* */
-      printf("PCG_BPX_refine |res|=%8.1e it=%3d\n", resi, iter);
-      bpx_free(&bpx);
-    }
-  /* OPTIONAL: estimate error */
-  FIDX *marker;
-  TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
-  err=error_est_lame_residual_t2(&msh2, &nodem, lambda, mu, marker, &set, 2);  
-  FUNCTION_FAILURE_HANDLE( err, error_est_lame_residual_t2, main);
-  free(marker); 
+      /* OPTIONAL: estimate error */
+      TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
+      err=lame_error_est_residual_t2(&msh2, &nodem, lambda, mu, 
+				     marker, &nrmarked, &globest, &set, 2);   
+      FUNCTION_FAILURE_HANDLE( err, lame_error_est_residual_t2, main);
+      free(marker); 
+    } /* end if (stop==0) */
 
   if(set.write_mesh!=0) /* write data for visualisation */
     {
@@ -544,6 +610,23 @@ int main(int argc, char *argv[])
 			    main);
     }   
 
+
+  err=vector_alloc( &dIdu, dim*vx_nr );
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+  err=vector_alloc( &psi, dim*vx_nr );
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+  err=vector_alloc( &dIdx, dim*vx_nr );
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+  err=vector_alloc( &dIdF, msh2.sp_nr );
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+  
+  /* set vectors to zero */
+  for (i=0; i<dim*vx_nr; i++)
+    {
+      psi.V[i]  =0.0;
+      dIdu.V[i] =0.0;
+      dIdx.V[i] =0.0;
+    }
    
   /* Compute value of pccrits and the rhs for the adjoint problem */
   err=lame_dIdu_t2(&msh2, &nodem, &dIdu, &Is, lambda, mu);
@@ -566,6 +649,17 @@ int main(int argc, char *argv[])
 
       coarse_mat_free(&K_inv);
     }
+  if (set.solver==3) /* variant 3: direct solve with UMFPACK */
+    {
+      /* need to re-assemble the matrix, because sparse_solve_UMFPACK
+	 modified it */
+      err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &nodem,
+			    lambda, mu, stiffening, 2, 2);
+      FUNCTION_FAILURE_HANDLE( err, assem_lame_tx, main);
+
+      err=sparse_solve_UMFPACK( &psi, &Ks[lvlm], &dIdu, &P); /* */
+      FUNCTION_FAILURE_HANDLE( err, sparse_solve_UMFPACK, main);
+    }
   if (set.solver==1) /* variant 2: PCG */
     {
       /* Compute psi by solving K^T*psi = dIdu */
@@ -621,6 +715,61 @@ int main(int argc, char *argv[])
   if (0==1)
     {
       struct vector dIdx_allaire, diff_allaire;
+      if (0==1)
+	{
+	  /* compute J=0.5*(u^T K u) + rhs^T u to check implementation
+	     of compliance functional */
+
+	  struct vector hlp;
+	  double J;
+
+	  err=vector_alloc( &hlp, nodem.len );
+	  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+
+
+	  for (i=0; i<rhs.len; i++)
+	    {
+	      rhs.V[i]  =0.0;
+	      hlp.V[i]  =0.0;
+	    }
+
+	  err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &hlp,
+				lambda, mu, stiffening, 2, 2);
+	  FUNCTION_FAILURE_HANDLE( err, assem_lame_tx_tx, main);
+
+	  for (i=0; i<rhs.len; i++)
+	    {
+	      hlp.V[i]  =0.0;
+	    }
+
+	  /* multiply hlp= K*nodem */
+	  err=sparse_mul_mat_vec(&Ks[lvlm], &nodem, &hlp);
+	  FUNCTION_FAILURE_HANDLE( err, sparse_mul_mat_vec, main);
+
+	  /* J=nodem'*(0.5*hlp+rhs) */
+	  J=0.0;
+	  for (i=0; i<rhs.len; i++)
+	    {
+	      J+=nodem.V[i]*(0.5*hlp.V[i]+rhs.V[i]); 
+	    }
+	  
+	  printf("J=%e\n",J);
+	  /* is OK, value also computed with FEMLAB:
+	     Domain integral 
+	      0.5*(sx_pn*ex_pn+sy_pn*ey_pn+2*sxy_pn*exy_pn)
+	         =1.021971e-4
+	     Boundary integral 
+              u*1+v*0
+                 =2.043942e-4
+	     so  J=1.021971e-4+2.043942e-4
+	       J = 3.0659e-04
+	     FEINS gave 
+               I = 3.0661644109e-04  (dIdu function)
+               J = 3.066164e-04      (0.5*u'*K*u+u'*rhs)
+	  */
+
+	}
+
 
       err=vector_alloc( &dIdx_allaire, dim*vx_nr );
       FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
@@ -640,21 +789,148 @@ int main(int argc, char *argv[])
 
       for (i=0; i<dIdx_allaire.len; i++)
 	{
-	  diff_allaire.V[i]=(50.0/68.0*dIdx_allaire.V[i]-dIdx.V[i]);
-	  /*printf("allaire/DAM[%5d] = %16.7e\n",(int) i,dIdx_allaire.V[i]/dIdx.V[i]);*/
+	  diff_allaire.V[i]=(/*50.0/68.0* */ 1.5*dIdx_allaire.V[i]-dIdx.V[i]);
+	  /*printf("allaire/DAM[%5d] = %16.7e\n", i,dIdx_allaire.V[i]/dIdx.V[i]);*/
 	}
       err=mesh_write_solution_vtk_t2( &msh2, &diff_allaire, dim, NULL, 0,
 				      30, "visual/Lame_DIDX_diff_allaire" );
       FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
 			       main);
+
+      if (0==1)
+	{
+	  /* restrict the derivatives back to the level 0 mesh, to be
+	     able to compare convergence */
+	  FIDX node,bd;
+
+	  /* redefine multilvl data */
+	  err=multilvl_init_tx( &msh2, dim, &ml, 2);
+	  FUNCTION_FAILURE_HANDLE( err, multilvl_init_tx, main);
+	  err=mg_init_tx( NULL, &msh2, &ml, &mg, NULL);
+	  FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
+
       
+	  /* copy stuff from fine mesh into mg.xl, mg.bl */
+	  for (i=0; i<dim*vx_nr; i++)
+	    {
+	      MLVLFINDENTRY(node, i, lvlm, ml);
+	      mg.xl[node]=dIdx.V[i];
+	      mg.bl[node]=dIdx_allaire.V[i];
+	    }
+
+	  /* restrict to coarsest mesh */
+	  err=mg_restrict_t2( &mg, lvlm, 0, mg.xl);
+	  FUNCTION_FAILURE_HANDLE( err, mg_restrict_t2, main);
+	  err=mg_restrict_t2( &mg, lvlm, 0, mg.bl);
+	  FUNCTION_FAILURE_HANDLE( err, mg_restrict_t2, main);
+
+	  /* interpolate back to fine mesh for plotting */
+	  err=mg_interpolate_t2( &mg, 0, lvlm, mg.xl);	
+	  FUNCTION_FAILURE_HANDLE( err, mg_interpolate_t2, main);  
+	  err=mg_interpolate_t2( &mg, 0, lvlm, mg.bl);	
+	  FUNCTION_FAILURE_HANDLE( err, mg_interpolate_t2, main);  
+	      
+	  /* copy back all boundary points */    
+	  for (bd=0; bd<msh2.bd_nr; bd++)
+	    {
+	      FIDX eg=msh2.bound[bd*msh2.bd_w+MCT2BDEDGE];
+
+	      /* work only on finest boundary edges */
+	      if (msh2.edge[eg*msh2.eg_w + MCT2EGCHL1] == -1)
+		{
+		  for (i=0; i<3; i++)
+		    {
+		      FIDX nodei=msh2.edge[eg*msh2.eg_w+MCT2EGNOD1+i];
+		      for (d=0; d<dim; d++)
+			{
+			  MLVLFINDENTRY(node, nodei+d*vx_nr, lvlm, ml);
+			  dIdx.V[nodei+d*vx_nr]        =mg.xl[node];
+			  dIdx_allaire.V[nodei+d*vx_nr]=mg.bl[node];
+
+			  /* re-compute difference */
+			  diff_allaire.V[nodei+d*vx_nr]=
+			    dIdx.V[nodei+d*vx_nr]
+			    -1.5*dIdx_allaire.V[nodei+d*vx_nr];
+			}
+		    }
+		}
+	    }
+
+	  /* write those to files */
+	  err=mesh_write_solution_vtk_t2( &msh2, &dIdx, dim, NULL, 0,
+					  26, "visual/Lame_coarseBD_DIDX" );
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+				   main);
+
+	  err=mesh_write_solution_vtk_t2( &msh2, &dIdx_allaire, dim, NULL, 0,
+					  34, "visual/Lame_coarseBD_DIDX_allaire" );
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+				   main);
+
+	  err=mesh_write_solution_vtk_t2( &msh2, &diff_allaire, dim, NULL, 0,
+					  39, "visual/Lame_coarseBD_DIDX_diff_allaire" );
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+				   main);
+
+
+	  multilvl_free(&ml);	
+	  mg_free(&mg);
+	} /* end restrict back to coarsest mesh */
+
+      if (1==1)
+	{
+	  double Rs[]={1.0, 2.0};
+	  /*
+	    plot "visual/plot_phi_DIDX_allaire_R0.txt" with lines,"visual/plot_phi_DIDX_R0.txt" with lines, "visual/plot_phi_diff_allaire_R0.txt" with lines
+	    
+	    plot "visual/plot_phi_DIDX_allaire_R1.txt" with lines,"visual/plot_phi_DIDX_R1.txt" with lines, "visual/plot_phi_diff_allaire_R1.txt" with lines
+	  */
+	  err=mesh_gnuplot_boundary_circles_t2( 2, Rs, 0.01, &msh2, &dIdx, dim, 1, 
+				    "visual/plot_phi_DIDX");
+	  FUNCTION_FAILURE_HANDLE( err, mesh_gnuplot_boundary_circles_t2, main);
+	  
+	  err=mesh_gnuplot_boundary_circles_t2( 2, Rs, 0.01, &msh2, &dIdx_allaire, dim, 1, 
+				    "visual/plot_phi_DIDX_allaire");
+	  FUNCTION_FAILURE_HANDLE( err, mesh_gnuplot_boundary_circles_t2, main);
+
+	  err=mesh_gnuplot_boundary_circles_t2( 2, Rs, 0.01, &msh2, &diff_allaire, dim, 1, 
+				    "visual/plot_phi_diff_allaire");
+	  FUNCTION_FAILURE_HANDLE( err, mesh_gnuplot_boundary_circles_t2, main);
+	}
+
+    
+      vector_free(&dIdx_allaire);
+      vector_free(&diff_allaire);
+    } /* end test derivatives Allaire et al paper */
+
+  /* test the Eppler conjecture */
+  if (0==1)
+    {
+      double Rs[]={1.0, 2.0};
+      struct vector sdifference;
+
+      err=vector_alloc( &sdifference, vx_nr );
+      FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+
+      err=lame_eppler_conjecture_2010_rem3_3( &msh2, lambda, mu,
+					      &nodem, &sdifference);
+      FUNCTION_FAILURE_HANDLE( err, assem_lame_tx_tx, main);
+
+      err=mesh_write_solution_vtk_t2( &msh2, NULL, 0, &sdifference, 1,
+				      31, "visual/Lame_eppler_sdifference" );
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+				   main);
+
+      err=mesh_gnuplot_boundary_circles_t2( 2, Rs, 0.01, &msh2, &sdifference, 1, 1, 
+				"visual/plot_phi_eppler_sdifference");
+      FUNCTION_FAILURE_HANDLE( err, mesh_gnuplot_boundary_circles_t2, main);
+
+      vector_free(&sdifference);
     }
 
 
 
 
-
-
   /* compute DIDF from DIDx */
   /*                       msh, insert, nI, dIdx,  dIdF, type */
   err=mesh_sseg_adjust_tx( &msh2, 0,     1, &dIdx, &dIdF, 2); 
@@ -663,24 +939,37 @@ int main(int argc, char *argv[])
 
     
   /* FINITE DIFFERENCE FOR dIdF_FD */
-  for (j=0; j<0*(msh2.sp_nr); j++)
+  if (0==1)
     {
-      double hlp=msh2.spar[j];
-      double h;
-      h= 1e-4;
+      double a,b,c;
 
-      /* +h */
-      msh2.spar[j] =hlp+h;
+      err=vector_alloc( &dIdx_FD, dim*vx_nr );
+      FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
 
-      err=mesh_sseg_adjust_tx( &msh2,  0,     0, NULL, NULL, 2);
-      FUNCTION_FAILURE_HANDLE( err, mesh_sseg_adjust_tx, main); 
+      for (j=0; j<0*(msh2.sp_nr); j++)
+	{
+	  double hlp=msh2.spar[j];
+	  double h;
+	  h= 1e-6;
 
-      err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &nodem,
-	lambda, mu, stiffening, 2, 2);
+	  /* +h */
+	  msh2.spar[j] =hlp+h;
+
+	  err=mesh_sseg_adjust_tx( &msh2,  0,     0, NULL, NULL, 2);
+	  FUNCTION_FAILURE_HANDLE( err, mesh_sseg_adjust_tx, main); 
+
+	  for (i=0; i<nodem.len; i++)
+	    {
+	      nodem.V[i]=0.0;
+	      rhs.V[i]  =0.0;
+	    }
+
+	  err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &nodem,
+				lambda, mu, stiffening, 2, 2);
 	{
 	  /* invert the stiffness matrix */
 	  err=coarse_mat_set( &Ks[lvlm], P.len, P.V,
-			  1, &K_inv );
+			      1, &K_inv );
 	  FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
 
 	  /* project rhs */
@@ -692,20 +981,26 @@ int main(int argc, char *argv[])
 	  coarse_mat_free(&K_inv);
 	}
 	Isp=0.0;
-      err=lame_dIdu_t2( &msh2, &nodem, &dIdu, &Isp, lambda, mu);
+	err=lame_dIdu_t2( &msh2, &nodem, &dIdu, &Isp, lambda, mu);
 
-      /* -h */
-      msh2.spar[j] =hlp-h;
+	/* -h */
+	msh2.spar[j] =hlp-h;
 
-      err=mesh_sseg_adjust_tx( &msh2,  0,     0, NULL, NULL, 2);
-      FUNCTION_FAILURE_HANDLE( err, mesh_sseg_adjust_tx, main); 
+	err=mesh_sseg_adjust_tx( &msh2,  0,     0, NULL, NULL, 2);
+	FUNCTION_FAILURE_HANDLE( err, mesh_sseg_adjust_tx, main); 
 
-      err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &nodem,
-	lambda, mu, stiffening, 2, 2);
+	for (i=0; i<nodem.len; i++)
+	  {
+	    nodem.V[i]=0.0;
+	    rhs.V[i]  =0.0;
+	  }
+
+	err=assem_lame_tx_tx( &Ks[lvlm], &P, &msh2, &rhs, &nodem,
+			      lambda, mu, stiffening, 2, 2);
 	{
 	  /* invert the stiffness matrix */
 	  err=coarse_mat_set( &Ks[lvlm], P.len, P.V,
-			  1, &K_inv );
+			      1, &K_inv );
 	  FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
 
 	  /* project rhs */
@@ -717,30 +1012,30 @@ int main(int argc, char *argv[])
 	  coarse_mat_free(&K_inv);
 	}
 	Ism=0.0;
-      err=lame_dIdu_t2( &msh2, &nodem, &dIdu, &Ism, lambda, mu);
+	err=lame_dIdu_t2( &msh2, &nodem, &dIdu, &Ism, lambda, mu);
 
-      /* +-0 */
-      msh2.spar[j] =hlp;
+	/* +-0 */
+	msh2.spar[j] =hlp;
       
-      /* auswerten FD */
-      dIdx_FD.V[j]=(Isp-Ism)/(2*h);
-    
-      double a,b,c;
-      a=dIdF.V[j];
-      b=dIdx_FD.V[j];
-      c=fabs((a-b)/b);
-      if (c>0.01)
-	{
-	  printf("%16.14f\t%16.14f\t%f\n", a, b, c);
+	/* auswerten FD */
+	dIdx_FD.V[j]=(Isp-Ism)/(2*h);
+	
+	a=dIdF.V[j];
+	b=dIdx_FD.V[j];
+	c=fabs((a-b)/b);
+	if (c>1e-6)
+	  {
+	    printf("par %3"dFIDX" \t adj=%16.14f \t FD=%16.14f \t rel=%f\n",  j, a, b, c);
+	  }
 	}
+      vector_free(&dIdx_FD);
+      /* FINITE DIFFERENCE IS DONE */
     }
-  /* FINITE DIFFERENCE IS DONE */
 
   /* write results to files */
   /* I */ 
   strcpy(buffer, argv[1]);
   strcat(buffer,"_crit");
-  FILE *out;
   /* open the file */
   out=fopen(buffer, "w");
   if (out==NULL)
@@ -779,7 +1074,6 @@ int main(int argc, char *argv[])
   vector_free(&dIdu);
   vector_free(&psi);
   vector_free(&dIdx);
-  vector_free(&dIdx_FD);
   vector_free(&dIdF);
 
   return SUCCESS;
